/**
  ******************************************************************************
  * @file    usbd_audio_core.c
  * @author  MCD Application Team
  * @version V1.1.0RC1
  * @date    27-January-2012
  * @brief   This file provides the high layer firmware functions to manage the 
  *          following functionalities of the USB Audio Class:
  *           - Initialization and Configuration of high and low layer
  *           - Enumeration as Audio Streaming Device
  *           - Audio Streaming data transfer
  *           - AudioControl requests management
  *           - Error management
  *           
  *  @verbatim
  *      
  *          ===================================================================      
  *                                Audio Class Driver Description
  *          =================================================================== 
  *           This driver manages the Audio Class 1.0 following the "USB Device Class Definition for
  *           Audio Devices V1.0 Mar 18, 98".
  *           This driver implements the following aspects of the specification:
  *             - Device descriptor management
  *             - Configuration descriptor management
  *             - Standard AC Interface Descriptor management
  *             - 1 Audio Streaming Interface (with single channel, PCM, Stereo mode)
  *             - 1 Audio Streaming Endpoint
  *             - 1 Audio Terminal Input (1 channel)
  *             - Audio Class-Specific AC Interfaces
  *             - Audio Class-Specific AS Interfaces
  *             - AudioControl Requests: only SET_CUR and GET_CUR requests are supported (for Mute)
  *             - Audio Feature Unit (limited to Mute control)
  *             - Audio Synchronization type: Asynchronous
  *             - Single fixed audio sampling rate (configurable in usbd_conf.h file)
  *          
  *           @note
  *            The Audio Class 1.0 is based on USB Specification 1.0 and thus supports only
  *            Low and Full speed modes and does not allow High Speed transfers.
  *            Please refer to "USB Device Class Definition for Audio Devices V1.0 Mar 18, 98"
  *            for more details.
  * 
  *           These aspects may be enriched or modified for a specific user application.
  *          
  *            This driver doesn't implement the following aspects of the specification 
  *            (but it is possible to manage these features with some modifications on this driver):
  *             - AudioControl Endpoint management
  *             - AudioControl requsests other than SET_CUR and GET_CUR
  *             - Abstraction layer for AudioControl requests (only Mute functionality is managed)
  *             - Audio Synchronization type: Adaptive
  *             - Audio Compression modules and interfaces
  *             - MIDI interfaces and modules
  *             - Mixer/Selector/Processing/Extension Units (Feature unit is limited to Mute control)
  *             - Any other application-specific modules
  *             - Multiple and Variable audio sampling rates
  *             - Out Streaming Endpoint/Interface (microphone)
  *      
  *  @endverbatim
  *                                  
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * FOR MORE INFORMATION PLEASE READ CAREFULLY THE LICENSE AGREEMENT FILE
  * LOCATED IN THE ROOT DIRECTORY OF THIS FIRMWARE PACKAGE.
  *
  * <h2><center>&copy; COPYRIGHT 2012 STMicroelectronics</center></h2>
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "stm32_audio.h"
#include "usbd_audio_core.h"
#include "stm32_audio_out_if.h"
#include "stm32_audio_in_if.h"

#if (AUDIO_SYNCHRO_SRC_ENABLED == 1)
 #ifdef USE_STM322xG_EVAL
  #include "stm32f2_resampling.h"
 #elif defined (USE_STM324xG_EVAL)
  #include "stm32f4_resampling.h"
 #elif defined(USE_STM32F4XX_DISCOVERY)
  #include "stm32f4_resampling.h"
 #endif /* USE_STM322xG_EVAL */
#endif /* AUDIO_SYNCHRO_SRC_ENABLED */

/** @addtogroup STM32_USB_OTG_DEVICE_LIBRARY
  * @{
  */


/** @defgroup usbd_audio 
  * @brief usbd core module
  * @{
  */ 

/** @defgroup usbd_audio_Private_TypesDefinitions
  * @{
  */ 
/**
  * @}
  */ 


/** @defgroup usbd_audio_Private_Defines
  * @{
  */ 
/**
  * @}
  */ 


/** @defgroup usbd_audio_Private_Macros
  * @{
  */ 
/**
  * @}
  */ 


/** @defgroup usbd_audio_Private_FunctionPrototypes
  * @{
  */

/*********************************************
   AUDIO Device library callbacks
 *********************************************/
static uint8_t  usbd_audio_Init        (void  *pdev, uint8_t cfgidx);
static uint8_t  usbd_audio_DeInit      (void  *pdev, uint8_t cfgidx);
static uint8_t  usbd_audio_Setup       (void  *pdev, USB_SETUP_REQ *req);
static uint8_t  usbd_audio_EP0_RxReady (void *pdev);
static uint8_t  usbd_audio_DataIn      (void *pdev, uint8_t epnum);
static uint8_t  usbd_audio_DataOut     (void *pdev, uint8_t epnum);
static uint8_t  usbd_audio_SOF         (void *pdev);

static uint8_t  usbd_audio_IN_Incplt   (void  *pdev);
static uint8_t  usbd_audio_OUT_Incplt  (void  *pdev);
static uint8_t  *USBD_audio_GetCfgDesc (uint8_t speed, uint16_t *length);

/*********************************************
   AUDIO Requests management functions
 *********************************************/
#if defined(USB_AUDIO_V1_0)
 static void AUDIO_Req_GetCurrent(void *pdev, USB_SETUP_REQ *req);
 static void AUDIO_Req_SetCurrent(void *pdev, USB_SETUP_REQ *req);
#elif defined(USB_AUDIO_V2_0)
 static void AUDIO_Req_Get(void *pdev, USB_SETUP_REQ *req);
 static void AUDIO_Req_Set(void *pdev, USB_SETUP_REQ *req);
#else
 #error "ERROR: Missing define of USB_AUDIO_V1_0 or USB_AUDIO_V2_0 in usbd_audio_core.h!"
#endif /* USB_AUDIO_V1_0 */

 
void usbd_audio_DescUpdate(uint32_t FBstate);
/**
  * @}
  */ 

/** @defgroup usbd_audio_Private_Variables
  * @{
  */ 
/* Main Buffer for Audio Data Out transfers and its relative pointers */
uint8_t  IsocOutBuff [TOTAL_OUT_BUF_SIZE]; /* This is the main buffer where all out audio data are stored */
uint8_t* IsocOutWrPtr = IsocOutBuff;       /* This is the pointer used by the write process (from host to device) */
uint8_t* IsocOutRdPtr = IsocOutBuff;       /* This is the pointer used by the read process (from buffers to I2S) */

uint8_t  OutEmptyFrameCount   = 0;
uint32_t IsocOutCurBitRes     = DEFAULT_OUT_BIT_RESOLUTION;  /* Number of bits per sample */
uint32_t IsocOutCurSampleFreq = DEFAULT_OUT_AUDIO_FREQ;      /* Current sampling rate for the Out audio samples in Hz */
uint32_t IsocOutCurPacketSze  = (DEFAULT_OUT_AUDIO_FREQ * (DEFAULT_OUT_BIT_RESOLUTION/8) * 2) / 1000;
uint32_t IsocOutPacketSze     = (DEFAULT_OUT_AUDIO_FREQ * (DEFAULT_OUT_BIT_RESOLUTION/8) * 2) / 1000;

BUFF_DESC_TypeDef IsocOutBufDesc[OUT_PACKET_NUM];
uint32_t IsocOutWrBufDescIdx  = 0;
uint32_t IsocOutRdBufDescIdx  = 0;
uint8_t  IsocOutWrState       = STATE_IDLE;
uint8_t  IsocOutRdState       = STATE_IDLE;


uint32_t currDistanceOut = 0; ///This variable should be declared static into AUDIO_CORR_Out_GetCurrent() function
uint32_t currDistanceIn  = 0; ///This variable should be declared static into AUDIO_CORR_In_GetCurrent() function

/* Main Buffer for Audio Data IN transfers and its relative pointers */
uint8_t  IsocInBuff [TOTAL_IN_BUF_SIZE];  /* This is the main buffer where all out audio data are stored */
uint8_t* IsocInWrPtr = IsocInBuff;        /* This is the pointer used by the write process (from host to device) */
uint8_t* IsocInRdPtr = IsocInBuff;        /* This is the pointer used by the read process (from buffers to I2S) */
uint8_t IsocInBuffDummy[AUDIO_IN_PACKET]; /* This dummy buffer with 0 values will be sent when there is no availble data */

uint8_t  InEmptyFrameCount   =  0;
uint32_t IsocInPacketSze     =  AUDIO_IN_PACKET;
uint32_t IsocInCurPacketSze  =  AUDIO_IN_PACKET;
uint32_t IsocInSampleFreq    =  DEFAULT_IN_AUDIO_FREQ;
uint32_t IsocInCurSampleFreq =  DEFAULT_IN_AUDIO_FREQ;

BUFF_DESC_TypeDef IsocInBufDesc[IN_PACKET_NUM];
uint32_t IsocInWrBufDescIdx  = 0;
uint32_t IsocInRdBufDescIdx  = 0;
uint8_t  IsocInWrState       = STATE_IDLE;
uint8_t  IsocInRdState       = STATE_IDLE;

/* Variables used for Feeback synchronization method */
#if (AUDIO_SYNCHRO_FEEDBACK_ENABLED == 1)
/* Current and theoric samples number (in integer and fractional parts) */
uint32_t IsocOutCurSampleNbrTb[4] = { 
  (DEFAULT_OUT_AUDIO_FREQ/1000)                                    /* Current Integer part of the samples number */
  ,DEFAULT_OUT_AUDIO_FREQ - ((DEFAULT_OUT_AUDIO_FREQ/1000) * 1000) /* Current Fractional part of the samples number */
  ,(DEFAULT_OUT_AUDIO_FREQ/1000)                                   /* Theoric Integer part of the samples number */
  ,DEFAULT_OUT_AUDIO_FREQ - ((DEFAULT_OUT_AUDIO_FREQ/1000) * 1000) /* Theoric Fractional part of the samples number */
};

/* Table holding the samples number in 10.14 format */
uint8_t pFBBuf[FB_PCKT_SZE] = {
   FEEDBACK_FF_FS_B2((DEFAULT_OUT_AUDIO_FREQ/1000), DEFAULT_OUT_AUDIO_FREQ - ((DEFAULT_OUT_AUDIO_FREQ/1000) * 1000)),
   FEEDBACK_FF_FS_B1((DEFAULT_OUT_AUDIO_FREQ/1000), DEFAULT_OUT_AUDIO_FREQ - ((DEFAULT_OUT_AUDIO_FREQ/1000) * 1000)),
   FEEDBACK_FF_FS_B0((DEFAULT_OUT_AUDIO_FREQ/1000), DEFAULT_OUT_AUDIO_FREQ - ((DEFAULT_OUT_AUDIO_FREQ/1000) * 1000))
};
#endif /* AUDIO_SYNCHRO_FEEDBACK_ENABLED */

/* Main Buffer for Audio Control Rrequests transfers and its relative variables */
uint8_t  AudioCtl[16];
uint8_t  AudioCtlCmd = 0;
uint32_t AudioCtlLen = 0;
uint8_t  AudioCtlUnit = 0;
uint8_t  AudioTrgtEpt = 0;

#if defined(USB_AUDIO_V2_0)
 uint8_t  MuteCtrl = 0;
#endif /* USB_AUDIO_V2_0 */


/////////////////////////////SRC Method 5
#if (AUDIO_SYNCHRO_SRC_ENABLED == 1)
 __IO uint32_t ratio = RATIO_1; 
 __IO uint32_t SrcOutSize = 0;
//static uint32_t SRCProcessing = 0;
#endif /* AUDIO_SYNCHRO_SRC_ENABLED */

static __IO uint32_t  usbd_audio_AltSet = 0;

/* USB AUDIO device Configuration Descriptor */
/*--------------------------------------------------------------------------------------------------------------------*/
/*                                          USB Audio V1.0 descriptor                                                 */
/*--------------------------------------------------------------------------------------------------------------------*/
static uint8_t usbd_audio_CfgDesc[AUDIO_CONFIG_DESC_SIZE] =
{
#if defined (USB_AUDIO_V1_0)  
  /* Configuration 1 */
  0x09,                                 /* bLength */
  USB_CONFIGURATION_DESCRIPTOR_TYPE,    /* bDescriptorType */
  LOBYTE(AUDIO_CONFIG_DESC_SIZE),       /* wTotalLength */
  HIBYTE(AUDIO_CONFIG_DESC_SIZE),      
#if (AUDIO_IN_ENABLED == 0)
  0x02,                                 /* bNumInterfaces */
#else
  0x03,                                 /* bNumInterfaces */
#endif /*AUDIO_IN_ENABLED  */
  0x01,                                 /* bConfigurationValue */
  0x00,                                 /* iConfiguration */
  0xC0,                                 /* bmAttributes  BUS Powred*/
  0x32,                                 /* bMaxPower = 100 mA*/
  /* 09 byte*/
  
  /*----------------------------------------- 
                         USB Speaker Audio Control Interface 
                                             -------------------------------------------------*/  
  /* USB Speaker Standard Interface descriptor */
  AUDIO_INTERFACE_DESC_SIZE,            /* bLength */
  USB_INTERFACE_DESCRIPTOR_TYPE,        /* bDescriptorType */
  0x00,                                 /* bInterfaceNumber */
  0x00,                                 /* bAlternateSetting */
  0x00,                                 /* bNumEndpoints */
  USB_DEVICE_CLASS_AUDIO,               /* bInterfaceClass */
  AUDIO_SUBCLASS_AUDIOCONTROL,          /* bInterfaceSubClass */
  AUDIO_PROTOCOL_UNDEFINED,             /* bInterfaceProtocol */
  0x00,                                 /* iInterface */
  /* 09 byte*/
  
  /* USB Speaker Class-specific AC Interface Descriptor */
#if (AUDIO_IN_ENABLED == 0)
  0x09,                                 /* bLength */
#else
  0x0A,                                 /* bLength */
#endif /* AUDIO_IN_ENABLED */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_CONTROL_HEADER,                 /* bDescriptorSubtype */
  0x00,          /* 1.00 */             /* bcdADC */
  0x01,
  0x27                                  /* wTotalLength = 39*/
#if (AUDIO_IN_ENABLED == 1)  
    + 31,                               /* wTotalLength = 39 + 31 */
#else
  ,
#endif /* AUDIO_IN_ENABLED */
   0,
#if (AUDIO_IN_ENABLED == 0)
  0x01,                                 /* bInCollection */
  0x01,                                 /* baInterfaceNr */
#else
  0x02,                                 /* bInCollection */
  0x01,                                 /* baInterfaceNr */
  0x02,                                 /* baInterfaceNr */
#endif /* AUDIO_IN_ENABLED */
  /* 9 or 10 bytes */
  
  /* USB Speaker Input Terminal Descriptor */
  AUDIO_INPUT_TERMINAL_DESC_SIZE,       /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_CONTROL_INPUT_TERMINAL,         /* bDescriptorSubtype */
  0x01,                                 /* bTerminal ID */
  0x01,                                 /* wTerminalType AUDIO_TERMINAL_USB_STREAMING   0x0101 */
  0x01,
  0x00,                                 /* bAssocTerminal */
  0x01,                                 /* bNrChannels */
  0x00,                                 /* wChannelConfig 0x0000  Mono */
  0x00,
  0x00,                                 /* iChannelNames */
  0x00,                                 /* iTerminal */
  /* 12 byte*/
  
  /* USB Speaker Audio Feature Unit Descriptor */
  0x09,                                 /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_CONTROL_FEATURE_UNIT,           /* bDescriptorSubtype */
  0x02,                                 /* bUnitID */
  0x01,                                 /* bSourceID */
  0x01,                                 /* bControlSize */
  AUDIO_CONTROL_MUTE,                   /* bmaControls(0) */
  0x00,                                 /* bmaControls(1) */
  0x00,                                 /* iTerminal */
  /* 09 byte*/
  
  /*USB Speaker Output Terminal Descriptor */
  0x09,                                 /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_CONTROL_OUTPUT_TERMINAL,        /* bDescriptorSubtype */
  0x03,                                 /* bTerminalID */
  0x01,                                 /* wTerminalType  0x0301*/
  0x03,
  0x00,                                 /* bAssocTerminal */
  0x02,                                 /* bSourceID */
  0x00,                                 /* iTerminal */
  /* 09 byte*/

#if (AUDIO_IN_ENABLED == 1)    
  /* USB Microphone Input Terminal Descriptor */
  AUDIO_INPUT_TERMINAL_DESC_SIZE,       /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_CONTROL_INPUT_TERMINAL,         /* bDescriptorSubtype */
  MIC_IN_TERMINAL_ID,                   /* bTerminal ID */
  0x01,                                 /* wTerminalType Generic Microphone   0x0201 */
  0x02,
  0x00,                                 /* bAssocTerminal */
  0x01,                                 /* bNrChannels */
  0x00,                                 /* wChannelConfig 0x0000  Mono */
  0x00,
  0x00,                                 /* iChannelNames */
  0x00,                                 /* iTerminal */
  /* 12 byte*/

  /* USB Microphone Audio Feature Unit Descriptor */
  0x09,                                 /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_CONTROL_FEATURE_UNIT,           /* bDescriptorSubtype */
  MIC_FU_ID,                            /* bUnitID */
  MIC_IN_TERMINAL_ID,                   /* bSourceID */
  0x01,                                 /* bControlSize */
  AUDIO_CONTROL_MUTE,                   /* bmaControls(0) */
  0x00,                                 /* bmaControls(1) */
  0x00,                                 /* iTerminal */
  /* 09 byte*/
  
  /*USB Microphone Output Terminal Descriptor */
  0x09,                                 /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_CONTROL_OUTPUT_TERMINAL,        /* bDescriptorSubtype */
  MIC_OUT_TERMINAL_ID,                  /* bTerminalID */
  0x01,                                 /* wTerminalType USB STREAMING 0x0101*/
  0x01,                                 
  0x00,                                 /* bAssocTerminal */ 
  MIC_FU_ID,                            /* bSourceID */
  0x00,                                 /* iTerminal */
  /* 09 byte*/
#endif /* AUDIO_IN_ENABLED   */

  /*----------------------------------------- 
                         USB Speaker Audio Streaming Interface 
                                             -------------------------------------------------*/
  /* USB Speaker Standard AS Interface Descriptor - Audio Streaming Zero Bandwith */
  /* Interface 1, Alternate Setting 0                                             */
  AUDIO_INTERFACE_DESC_SIZE,            /* bLength */
  USB_INTERFACE_DESCRIPTOR_TYPE,        /* bDescriptorType */
  0x01,                                 /* bInterfaceNumber */
  0x00,                                 /* bAlternateSetting */
  0x00,                                 /* bNumEndpoints */
  USB_DEVICE_CLASS_AUDIO,               /* bInterfaceClass */
  AUDIO_SUBCLASS_AUDIOSTREAMING,        /* bInterfaceSubClass */
  AUDIO_PROTOCOL_UNDEFINED,             /* bInterfaceProtocol */
  0x00,                                 /* iInterface */
  /* 09 byte*/
  
  /* USB Speaker Standard AS Interface Descriptor - Audio Streaming Operational */
  /* Interface 1, Alternate Setting 1                                           */
  AUDIO_INTERFACE_DESC_SIZE,            /* bLength */
  USB_INTERFACE_DESCRIPTOR_TYPE,        /* bDescriptorType */
  USB_SPEAKER_MAIN_IF_NUM,              /* bInterfaceNumber */  
  USB_SPEAKER_MAIN_IF_ALT_NUM,          /* bAlternateSetting */
  0x02,                                 /* bNumEndpoints */  /* >>> Updated when feedback pipe is active: "0x02" */
  USB_DEVICE_CLASS_AUDIO,               /* bInterfaceClass */
  AUDIO_SUBCLASS_AUDIOSTREAMING,        /* bInterfaceSubClass */
  AUDIO_PROTOCOL_UNDEFINED,             /* bInterfaceProtocol */
  0x00,                                 /* iInterface */
  /* 09 byte*/
  
  /* USB Speaker Audio Streaming Class-Specific Interface Descriptor */
  AUDIO_STREAMING_INTERFACE_DESC_SIZE,  /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_STREAMING_GENERAL,              /* bDescriptorSubtype */
  0x01,                                 /* bTerminalLink */
  0x01,                                 /* bDelay */
  0x01,                                 /* wFormatTag AUDIO_FORMAT_PCM  0x0001*/
  0x00,
  /* 07 byte*/
  
  /* USB Speaker Audio Type I Format Interface Descriptor */
  (0x08 + (3 * SUPPORTED_FREQ_NBR)),                           /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_STREAMING_FORMAT_TYPE,          /* bDescriptorSubtype */
  AUDIO_FORMAT_TYPE_I,                  /* bFormatType */   
  0x02,                                 /* bNrChannels */
  0x02,                                 /* bSubFrameSize :  2 Bytes per frame (16bits) */
  16,                                   /* bBitResolution (16-bits per sample) */ 
  SUPPORTED_FREQ_NBR,                   /* bSamFreqType number of supported frequencies */ 
  SAMPLE_FREQ(DEFAULT_OUT_AUDIO_FREQ),  /* Audio sampling frequency coded on 3 bytes */
#if (SUPPORTED_FREQ_NBR > 1)
  SAMPLE_FREQ(USBD_AUDIO_FREQ_2),       /* Audio sampling frequency coded on 3 bytes */
#endif
#if (SUPPORTED_FREQ_NBR > 2)
  SAMPLE_FREQ(USBD_AUDIO_FREQ_3),       /* Audio sampling frequency coded on 3 bytes */
#endif
#if (SUPPORTED_FREQ_NBR > 3)
  SAMPLE_FREQ(USBD_AUDIO_FREQ_4),       /* Audio sampling frequency coded on 3 bytes */
#endif
#if (SUPPORTED_FREQ_NBR > 4)
 #error "Error: Max nbr of frequencies exceeded, you have to modify the conf descriptor in usbd_audio_core.c file!"
#endif
 /* 11 + 3*N byte */
  
  /* USB Speaker Streaming Standard Endpoint Descriptor */
  AUDIO_STANDARD_ENDPOINT_DESC_SIZE,    /* bLength */
  USB_ENDPOINT_DESCRIPTOR_TYPE,         /* bDescriptorType */
  AUDIO_OUT_EP,                         /* bEndpointAddress 1 out endpoint*/
  0x05,                                 /* bmAttributes: Isochrnous | Asynchronous */ 
  LOBYTE(AUDIO_OUT_MPS),                /* wMaxPacketSize */ 
  HIBYTE(AUDIO_OUT_MPS), 
#ifdef USE_USB_OTG_HS 
  0x04,                                 /* bInterval must be 1*/ ///4 for HS mode ??
#else
  0x01,                                 /* bInterval must be 1*/
#endif /* USE_USB_OTG_HS  */
  0x00,                                 /* bRefresh must be 0 */
  0x00,                                  /* bSynchAddress */  /* >>> Updated when Feedback Pipe is active: "AUDIO_FB_EP" */
  /* 09 byte*/
  
  /* Endpoint - Audio Streaming Descriptor*/
  AUDIO_STREAMING_ENDPOINT_DESC_SIZE,   /* bLength */
  AUDIO_ENDPOINT_DESCRIPTOR_TYPE,       /* bDescriptorType */
  AUDIO_ENDPOINT_GENERAL,               /* bDescriptor */
#if (SUPPORTED_FREQ_NBR > 1)  
  0x01,                                 /* bmAttributes: Freq selection */
#else
  0x00,                                 /* bmAttributes: None */
#endif /* (SUPPORTED_FREQ_NBR > 1)  */
  0x00,                                 /* bLockDelayUnits */
  0x00,                                 /* wLockDelay */
  0x00,
  /* 07 byte*/

  /* >>> This descriptor is ignored when Feedback Pipe is not active */
  
#if (AUDIO_SYNCHRO_FEEDBACK_ENABLED == 1)
  /* Feedback Endpoint - Standard Descriptor */
  0x09,                                 /* bLength */
  USB_ENDPOINT_DESCRIPTOR_TYPE,         /* bDescriptorType */
  AUDIO_FB_EP,                          /* bEndpointAddress */
  0x11,                                 /* bmAttributes: Isochrounous | Feedback */ 
  0x03,                                 
  0x00,                                 /* wMaxPacketSize in Bytes: 3 in FS mode, 4 in HS mode */
  0x01,                                 /* bInterval */
  FEEDBACK_REFRESH,                     /* bRefresh: 2^FEEDBACK_REFRESH ms */
  0x00,                                  /* bSynchAddress */
  /* 09 byte*/
#endif /* AUDIO_SYNCHRO_FEEDBACK_ENABLED */

#if (AUDIO_IN_ENABLED == 1) 
  /*----------------------------------------- 
                         USB Microphone Audio Streaming Interface 
                                             -------------------------------------------------*/
  /* USB Microphone Standard AS Interface Descriptor - Audio Streaming Zero Bandwith */
  /* Interface 2, Alternate Setting 0                                             */
  AUDIO_INTERFACE_DESC_SIZE,            /* bLength */
  USB_INTERFACE_DESCRIPTOR_TYPE,        /* bDescriptorType */
  0x02,                                 /* bInterfaceNumber */
  0x00,                                 /* bAlternateSetting */
  0x00,                                 /* bNumEndpoints */
  USB_DEVICE_CLASS_AUDIO,               /* bInterfaceClass */
  AUDIO_SUBCLASS_AUDIOSTREAMING,        /* bInterfaceSubClass */
  AUDIO_PROTOCOL_UNDEFINED,             /* bInterfaceProtocol */
  0x00,                                 /* iInterface */
  /* 09 byte*/
  
  /* USB Microphone Standard AS Interface Descriptor - Audio Streaming Operational */
  /* Interface 2, Alternate Setting 1                                           */
  AUDIO_INTERFACE_DESC_SIZE,            /* bLength */
  USB_INTERFACE_DESCRIPTOR_TYPE,        /* bDescriptorType */
  0x02,                                 /* bInterfaceNumber */
  0x01,                                 /* bAlternateSetting */
  0x01,                                 /* bNumEndpoints */
  USB_DEVICE_CLASS_AUDIO,               /* bInterfaceClass */
  AUDIO_SUBCLASS_AUDIOSTREAMING,        /* bInterfaceSubClass */
  AUDIO_PROTOCOL_UNDEFINED,             /* bInterfaceProtocol */
  0x00,                                 /* iInterface */
  /* 09 byte*/
  
  /* USB Microphone Audio Streaming Class-Specific Interface Descriptor */
  AUDIO_STREAMING_INTERFACE_DESC_SIZE,  /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_STREAMING_GENERAL,              /* bDescriptorSubtype */
  MIC_OUT_TERMINAL_ID,                  /* bTerminalLink */
  0x01,                                 /* bDelay */
  0x01,                                 /* wFormatTag AUDIO_FORMAT_PCM  0x0001*/
  0x00,
  /* 07 byte*/
  
  /* USB Speaker Audio Type I Format Interface Descriptor */
  (0x08 + (3 /** SUPPORTED_FREQ_NBR*/)),    /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_STREAMING_FORMAT_TYPE,          /* bDescriptorSubtype */
  AUDIO_FORMAT_TYPE_I,                  /* bFormatType */   
  0x01,                                 /* bNrChannels */
#if (DEFAULT_IN_BIT_RESOLUTION <= 8)
  0x01,                                 /* bSubFrameSize: 1 byte */
#elif (DEFAULT_IN_BIT_RESOLUTION <= 16)
  0x02,                                 /* bSubFrameSize: 2 bytes */
#elif (DEFAULT_IN_BIT_RESOLUTION <= 24)
  0x03,                                 /* bSubFrameSize: 3 bytes */
#elif (DEFAULT_IN_BIT_RESOLUTION <= 32)
  0x04,                                 /* bSubFrameSize: 4 bytes */
#endif
  DEFAULT_IN_BIT_RESOLUTION,            /* bBitResolution (effective bits per sample) */ 
  1, //SUPPORTED_FREQ_NBR,                   /* bSamFreqType number of supported frequencies */ 
  SAMPLE_FREQ(DEFAULT_IN_AUDIO_FREQ),   /* Audio sampling frequency coded on 3 bytes */
  /* 8 + 3*N byte*/
  
  /* USB Microphone Streaming Standard Endpoint Descriptor */
  AUDIO_STANDARD_ENDPOINT_DESC_SIZE,    /* bLength */
  USB_ENDPOINT_DESCRIPTOR_TYPE,         /* bDescriptorType */
  AUDIO_IN_EP,                          /* bEndpointAddress 1 In endpoint*/
  0x05,                                 /* bmAttributes: Isochrnous | Asynchronous */ 
  LOBYTE(AUDIO_IN_MPS),                 /* wMaxPacketSize */ 
  HIBYTE(AUDIO_IN_MPS), 
#ifdef USE_USB_OTG_HS 
  0x04,                                 /* bInterval must be 1*/ ///4 for HS mode ??
#else
  0x01,                                 /* bInterval must be 1*/
#endif /* USE_USB_OTG_HS  */
  0x00,                                 /* bRefresh must be 0 */
  0x00,                                 /* bSynchAddress */
  /* 09 byte*/
  
  /* Endpoint - Audio Streaming Descriptor*/
  AUDIO_STREAMING_ENDPOINT_DESC_SIZE,   /* bLength */
  AUDIO_ENDPOINT_DESCRIPTOR_TYPE,       /* bDescriptorType */
  AUDIO_ENDPOINT_GENERAL,               /* bDescriptor */
  //#if (SUPPORTED_FREQ_NBR > 1)  
  //  0x01,                                 /* bmAttributes: Freq selection */
  //#else
  0x00,                                 /* bmAttributes: None */
  //#endif /* (SUPPORTED_FREQ_NBR > 1)  */
  0x00,                                 /* bLockDelayUnits */
  0x00,                                 /* wLockDelay */
  0x00,
  /* 07 byte*/    
  
#endif /* AUDIO_IN_ENABLED */  
  
  
  
  
  ///-----------------------------------------------------------------------------------------------------------------
   
    
/*--------------------------------------------------------------------------------------------------------------------*/
/*                                          USB Audio V2.0 descriptor                                                 */
/*              Not yet tested !                                                                                      */
/*--------------------------------------------------------------------------------------------------------------------*/
#elif defined(USB_AUDIO_V2_0)
  /* Configuration 1 */
  0x09,                                 /* bLength */
  USB_CONFIGURATION_DESCRIPTOR_TYPE,    /* bDescriptorType */
  LOBYTE(AUDIO_CONFIG_DESC_SIZE),       /* wTotalLength */
  HIBYTE(AUDIO_CONFIG_DESC_SIZE),      
  0x02,                                 /* bNumInterfaces */
  0x01,                                 /* bConfigurationValue */
  0x00,                                 /* iConfiguration */
  0xC0,                                 /* bmAttributes  BUS Powred*/
  0x32,                                 /* bMaxPower = 100 mA*/
  /* 09 byte*/

  /* USB Speaker Standard Interface Association Descriptor */
  0x08,                                 /* bLength */
  USB_INTERFACE_ASSOCIATION_TYPE,       /* bDescriptorType */
  0x00,                                 /* bFirstInterface: the first interface associated with this function */
  0x02,                                 /* bInterfaceCount: Number of contiguous interfaces 
                                           that are associated with this function */
  USB_DEVICE_CLASS_AUDIO,               /* bFunctionClass: Class Audio 0x01 */
  0x00,                                 /* bFunctionSubClass: not used */
  0x20,                                 /* bFunctionProtocol: USB Audio 2.0 */
  0x00,                                 /* iFunction */
  /* 08 byte*/

  /*----------------------------------------- 
                         USB Speaker Audio Control Interface 
                                             -------------------------------------------------*/
  /* USB Speaker Standard AC Interface Descriptor */
  AUDIO_INTERFACE_DESC_SIZE,            /* bLength */
  USB_INTERFACE_DESCRIPTOR_TYPE,        /* bDescriptorType */
  0x00,                                 /* bInterfaceNumber */
  0x00,                                 /* bAlternateSetting */
  0x00,                                 /* bNumEndpoints */
  USB_DEVICE_CLASS_AUDIO,               /* bInterfaceClass */
  AUDIO_SUBCLASS_AUDIOCONTROL,          /* bInterfaceSubClass */
  0x20,                                 /* bInterfaceProtocol */
  0x00,                                 /* iInterface */
  /* 09 byte*/  

  /* USB Speaker Class-specific AC Interface Descriptor */
  AUDIO_INTERFACE_DESC_SIZE,            /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_CONTROL_HEADER,                 /* bDescriptorSubtype */
  0x00,          /* 2.00 */             /* bcdADC */
  0x02,
  AUDIO_CATEGORY_DESKTOP_SPEAKER,       /* bCategory */
  0x3C,                                 /* wTotalLength total size of this descriptor 
                                           and all Clock, Unit and Terminal descriptors 
                                           = 9 + 8 + 17 + 14 + 12 */
  0x00,
  0x00,                                 /* bmControls: D1..0: Latency Control */
  /* 09 byte*/

  /* USB Speaker Clock Descriptor */
  0x08,                                 /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  0x0A,                                 /* bDescriptorSubtype: CLOCK_SOURCE */
  OUT_CLOCK_ID,                         /* bClockID */
  0x01,                                 /* bmAttributes: Internal fixed Clock */
  0x01,                                 /* bmControls: Clock Frequency Control (r) */
  0x00,                                 /* bAssocTerminal: None */
  0x00,                                 /* iClockSource */
  /* 08 byte*/  

  /* USB Speaker Input Terminal Descriptor */
  0x11,                                 /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_CONTROL_INPUT_TERMINAL,         /* bDescriptorSubtype */
  IN_TERMINAL_ID,                       /* bTerminal ID */
  0x01,                                 /* wTerminalType AUDIO_TERMINAL_USB_STREAMING   0x0101 */
  0x01,
  0x00,                                 /* bAssocTerminal: No association */
  OUT_CLOCK_ID,                         /* bCSourceID */
  0x01,                                 /* bNrChannels */
  0x01,                                 /* bmChannelConfig: Left Front */
  0x00,
  0x00,
  0x00,                                 
  0x00,                                 /* iChannelNames */
  0x00,                                 /* bmControls: No controls supported */
  0x00,                                 
  0x00,                                 /* iTerminal */
  /* 17 byte*/

  /* USB Speaker Audio Feature Unit Descriptor */
  0x0E,                                 /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_CONTROL_FEATURE_UNIT,           /* bDescriptorSubtype */
  OUT_FU_ID,                            /* bUnitID */
  IN_TERMINAL_ID,                       /* bSourceID */
  0x01,                                 /* bmaControls: Mute control (r) */ ///modify to 0x03 (rw)
  0x00,                                 
  0x00,                                 
  0x00,                                 
  0x00,                                 /* bmaControls: None */
  0x00,                                 
  0x00,                                 
  0x00,                                 
  0x00,                                 /* iFeature */
  /* 15 byte*/
  
  /*USB Speaker Output Terminal Descriptor */
  0x0C,                                 /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_CONTROL_OUTPUT_TERMINAL,        /* bDescriptorSubtype */
  OUT_TERMINAL_ID,                      /* bTerminalID */
  0x01,                                 /* wTerminalType  0x0301; Generic Speaker */
  0x03,
  0x00,                                 /* bAssocTerminal: No association */
  OUT_FU_ID,                            /* bSourceID */
  OUT_CLOCK_ID,                         /* bCSourceID */
  0x00,                                 /* bmControls: No controls supported */
  0x00,                                 
  0x00,                                 /* iTerminal */
  /* 12 byte*/


  /*----------------------------------------- 
                         USB Speaker Audio Streaming Interface 
                                             -------------------------------------------------*/

  /* USB Speaker Standard AS Interface Descriptor - Audio Streaming Zero Bandwith */
  /* Interface 1, Alternate Setting 0                                             */
  AUDIO_INTERFACE_DESC_SIZE,            /* bLength */
  USB_INTERFACE_DESCRIPTOR_TYPE,        /* bDescriptorType */
  0x01,                                 /* bInterfaceNumber */
  0x00,                                 /* bAlternateSetting */
  0x00,                                 /* bNumEndpoints */
  USB_DEVICE_CLASS_AUDIO,               /* bInterfaceClass */
  AUDIO_SUBCLASS_AUDIOSTREAMING,        /* bInterfaceSubClass */
  0x20,                                 /* bInterfaceProtocol */
  0x00,                                 /* iInterface */
  /* 09 byte*/

  /* USB Speaker Standard AS Interface Descriptor - Audio Streaming Operational */
  /* Interface 1, Alternate Setting 1                                           */
  AUDIO_INTERFACE_DESC_SIZE,            /* bLength */
  USB_INTERFACE_DESCRIPTOR_TYPE,        /* bDescriptorType */
  0x01,                                 /* bInterfaceNumber */
  0x01,                                 /* bAlternateSetting */
  #ifndef AUDIO_SYNCHRO_FEEDBACK_ENABLED  
  0x01,                                 /* bNumEndpoints */
  #else
  0x02,                                 /* bNumEndpoints */
  #endif /* AUDIO_SYNCHRO_FEEDBACK_ENABLED */
  USB_DEVICE_CLASS_AUDIO,               /* bInterfaceClass */
  AUDIO_SUBCLASS_AUDIOSTREAMING,        /* bInterfaceSubClass */
  0x20,                                 /* bInterfaceProtocol */
  0x00,                                 /* iInterface */
  /* 09 byte*/

  /* USB Speaker Class-Specific AS Interface Descriptor */
  0x10,                                 /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_STREAMING_GENERAL,              /* bDescriptorSubtype */
  IN_TERMINAL_ID,                       /* bTerminalLink */ 
  0x00,                                 /* bmControls: None */
  0x01,                                 /* bFormatType: FORMAT_TYPE_I */
  0x01,                                 /* bmFormats: PCM */
  0x00,
  0X00,
  0x00,                                 
  0x01,                                 /* bNrChannels */
  0x01,                                 /* bmChannelConfig: Spatial location Front Left */
  0x00,
  0x00,
  0x00,
  0x00,                                 /* iChannelNames */
  /* 16 byte*/

  /* USB Speaker Class-Specific AS Format Type Descriptor */
  0x06,                                 /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_STREAMING_FORMAT_TYPE,          /* bDescriptorSubtype */
  AUDIO_FORMAT_TYPE_I,                  /* bFormatType */   
  0x02,                                 /* bSubslotSize: 16-bit */
  0x10,                                 /* bBitResolution (16-bits per sample) */   
  /* 6 bytes */
  
  /* USB Speaker Standard AS Isochronous Data Endpoint Descriptor */
  0x07,                                 /* bLength */
  USB_ENDPOINT_DESCRIPTOR_TYPE,         /* bDescriptorType */
  AUDIO_OUT_EP,                         /* bEndpointAddress out endpoint*/
  0x05,                                 /* bmAttributes: Isochrnous | Asynchronous | Data Endpoint */ 
  0xFF,
  0x03,                                 /* wMaxPacketSize: 1023 bytes */
  0x01,                                 /* bInterval must be 1*/
  /* 07 byte*/
  
  /* USB Speaker Class-Specific AS Isochronous Data Endpoint Descriptor */
  0x08,                                 /* bLength */
  AUDIO_ENDPOINT_DESCRIPTOR_TYPE,       /* bDescriptorType */
  AUDIO_ENDPOINT_GENERAL,               /* bDescriptor */
  0x00,                                 /* bmAttributes */
  0x00,                                 /* bmControls */
  0x00,                                 /* bLockDelayUnits */
  0x00,                                 /* wLockDelay */
  0x00
  /* 08 byte*/
  
#ifdef AUDIO_SYNCHRO_FEEDBACK_ENABLED
  /* Feedback Endpoint - Standard Descriptor */
  ,0x07,                                /* bLength */
  USB_ENDPOINT_DESCRIPTOR_TYPE,         /* bDescriptorType */
  AUDIO_FB_EP,                          /* bEndpointAddress */
  0x11,                                 /* bmAttributes: Isochrounous | Feedback */ 
  0x04,                                 
  0x00,                                 /* wMaxPacketSize in Bytes: 3 in FS mode, 4 in HS mode */
  0x01                                  /* bInterval */
  /* 07 byte*/
#endif /* AUDIO_SYNCHRO_FEEDBACK_ENABLED */

#endif /* USB_AUDIO_V1_0 */
};

#ifdef USE_USB_OTG_HS
uint8_t usbd_audio_OtherCfgDesc[AUDIO_CONFIG_DESC_SIZE] =
{
#if defined (USB_AUDIO_V1_0)  
  /* Configuration 1 */
  0x09,                                 /* bLength */
  USB_CONFIGURATION_DESCRIPTOR_TYPE,    /* bDescriptorType */
  LOBYTE(AUDIO_CONFIG_DESC_SIZE),       /* wTotalLength */
  HIBYTE(AUDIO_CONFIG_DESC_SIZE),      
  0x02,                                 /* bNumInterfaces */
  0x01,                                 /* bConfigurationValue */
  0x00,                                 /* iConfiguration */
  0xC0,                                 /* bmAttributes  BUS Powred*/
  0x32,                                 /* bMaxPower = 100 mA*/
  /* 09 byte*/
  
  /* USB Speaker Standard interface descriptor */
  AUDIO_INTERFACE_DESC_SIZE,            /* bLength */
  USB_INTERFACE_DESCRIPTOR_TYPE,        /* bDescriptorType */
  0x00,                                 /* bInterfaceNumber */
  0x00,                                 /* bAlternateSetting */
  0x00,                                 /* bNumEndpoints */
  USB_DEVICE_CLASS_AUDIO,               /* bInterfaceClass */
  AUDIO_SUBCLASS_AUDIOCONTROL,          /* bInterfaceSubClass */
  AUDIO_PROTOCOL_UNDEFINED,             /* bInterfaceProtocol */
  0x00,                                 /* iInterface */
  /* 09 byte*/
  
  /* USB Speaker Class-specific AC Interface Descriptor */
  AUDIO_INTERFACE_DESC_SIZE,            /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_CONTROL_HEADER,                 /* bDescriptorSubtype */
  0x00,          /* 1.00 */             /* bcdADC */
  0x01,
  0x27,                                 /* wTotalLength = 39*/
  0x00,
  0x01,                                 /* bInCollection */
  0x01,                                 /* baInterfaceNr */
  /* 09 byte*/
  
  /* USB Speaker Input Terminal Descriptor */
  AUDIO_INPUT_TERMINAL_DESC_SIZE,       /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_CONTROL_INPUT_TERMINAL,         /* bDescriptorSubtype */
  0x01,                                 /* bTerminal ID */
  0x01,                                 /* wTerminalType AUDIO_TERMINAL_USB_STREAMING   0x0101 */
  0x01,
  0x00,                                 /* bAssocTerminal */
  0x01,                                 /* bNrChannels */
  0x00,                                 /* wChannelConfig 0x0000  Mono */
  0x00,
  0x00,                                 /* iChannelNames */
  0x00,                                 /* iTerminal */
  /* 12 byte*/
  
  /* USB Speaker Audio Feature Unit Descriptor */
  0x09,                                 /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_CONTROL_FEATURE_UNIT,           /* bDescriptorSubtype */
  AUDIO_OUT_STREAMING_CTRL,             /* bUnitID */
  0x01,                                 /* bSourceID */
  0x01,                                 /* bControlSize */
  AUDIO_CONTROL_MUTE,                   /* bmaControls(0) */
  0x00,                                 /* bmaControls(1) */
  0x00,                                 /* iTerminal */
  /* 09 byte*/
  
  /*USB Speaker Output Terminal Descriptor */
  0x09,                                 /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_CONTROL_OUTPUT_TERMINAL,        /* bDescriptorSubtype */
  0x03,                                 /* bTerminalID */
  0x01,                                 /* wTerminalType  0x0301*/
  0x03,
  0x00,                                 /* bAssocTerminal */
  0x02,                                 /* bSourceID */
  0x00,                                 /* iTerminal */
  /* 09 byte*/
  
  /* USB Speaker Standard AS Interface Descriptor - Audio Streaming Zero Bandwith */
  /* Interface 1, Alternate Setting 0                                             */
  AUDIO_INTERFACE_DESC_SIZE,            /* bLength */
  USB_INTERFACE_DESCRIPTOR_TYPE,        /* bDescriptorType */
  0x01,                                 /* bInterfaceNumber */
  0x00,                                 /* bAlternateSetting */
  0x00,                                 /* bNumEndpoints */
  USB_DEVICE_CLASS_AUDIO,               /* bInterfaceClass */
  AUDIO_SUBCLASS_AUDIOSTREAMING,        /* bInterfaceSubClass */
  AUDIO_PROTOCOL_UNDEFINED,             /* bInterfaceProtocol */
  0x00,                                 /* iInterface */
  /* 09 byte*/
  
  /* USB Speaker Standard AS Interface Descriptor - Audio Streaming Operational */
  /* Interface 1, Alternate Setting 1                                           */
  AUDIO_INTERFACE_DESC_SIZE,            /* bLength */
  USB_INTERFACE_DESCRIPTOR_TYPE,        /* bDescriptorType */
  0x01,                                 /* bInterfaceNumber */
  0x01,                                 /* bAlternateSetting */
#if (AUDIO_SYNCHRO_FEEDBACK_ENABLED == 1)  
  0x01,                                 /* bNumEndpoints */
#else
  0x02,                                 /* bNumEndpoints */
#endif /* AUDIO_SYNCHRO_FEEDBACK_ENABLED */
  USB_DEVICE_CLASS_AUDIO,               /* bInterfaceClass */
  AUDIO_SUBCLASS_AUDIOSTREAMING,        /* bInterfaceSubClass */
  AUDIO_PROTOCOL_UNDEFINED,             /* bInterfaceProtocol */
  0x00,                                 /* iInterface */
  /* 09 byte*/
  
  /* USB Speaker Audio Streaming Interface Descriptor */
  AUDIO_STREAMING_INTERFACE_DESC_SIZE,  /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_STREAMING_GENERAL,              /* bDescriptorSubtype */
  0x01,                                 /* bTerminalLink */
  0x01,                                 /* bDelay */
  0x01,                                 /* wFormatTag AUDIO_FORMAT_PCM  0x0001*/
  0x00,
  /* 07 byte*/
  
  /* USB Speaker Audio Type I Format Interface Descriptor */
  (0x08 + (3 * SUPPORTED_FREQ_NBR)),                           /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_STREAMING_FORMAT_TYPE,          /* bDescriptorSubtype */
  AUDIO_FORMAT_TYPE_I,                  /* bFormatType */   
  0x02,                                 /* bNrChannels */
  0x02,                                 /* bSubFrameSize :  2 Bytes per frame (16bits) */
  16,                                   /* bBitResolution (16-bits per sample) */ 
  SUPPORTED_FREQ_NBR,                   /* bSamFreqType number of supported frequencies */ 
  SAMPLE_FREQ(DEFAULT_OUT_AUDIO_FREQ),  /* Audio sampling frequency coded on 3 bytes */
#if (SUPPORTED_FREQ_NBR > 1)
  SAMPLE_FREQ(USBD_AUDIO_FREQ_2),       /* Audio sampling frequency coded on 3 bytes */
#endif
#if (SUPPORTED_FREQ_NBR > 2)
  SAMPLE_FREQ(USBD_AUDIO_FREQ_3),       /* Audio sampling frequency coded on 3 bytes */
#endif
#if (SUPPORTED_FREQ_NBR > 3)
  SAMPLE_FREQ(USBD_AUDIO_FREQ_4),       /* Audio sampling frequency coded on 3 bytes */
#endif
#if (SUPPORTED_FREQ_NBR > 4)
 #error "Error: Max nbr of frequencies exceeded, you have to modify the conf descriptor in usbd_audio_core.c file!"
#endif
/* 11 + 3*N byte*/
  
  /* Endpoint Data Out - Standard Descriptor */
  AUDIO_STANDARD_ENDPOINT_DESC_SIZE,    /* bLength */
  USB_ENDPOINT_DESCRIPTOR_TYPE,         /* bDescriptorType */
  AUDIO_OUT_EP,                         /* bEndpointAddress 1 out endpoint*/
  0x05,                                 /* bmAttributes: Isochrnous | Asynchronous */ 
  0xFF,
  0x03,                                 /* wMaxPacketSize: 1023 bytes */
  0x01,                                 /* bInterval must be 1*/
  0x00,                                 /* bRefresh must be 0 */
#if (AUDIO_SYNCHRO_FEEDBACK_ENABLED == 1)
  AUDIO_FB_EP,                          /* bSynchAddress */
 #else
  0x00,                                  /* bSynchAddress */
 #endif /* AUDIO_SYNCHRO_FEEDBACK_ENABLED */
  /* 09 byte*/
  
  /* Endpoint - Audio Streaming Descriptor*/
  AUDIO_STREAMING_ENDPOINT_DESC_SIZE,   /* bLength */
  AUDIO_ENDPOINT_DESCRIPTOR_TYPE,       /* bDescriptorType */
  AUDIO_ENDPOINT_GENERAL,               /* bDescriptor */
 #if (SUPPORTED_FREQ_NBR > 1)  
  0x01,                                 /* bmAttributes: Freq selection */
 #else
  0x00,                                 /* bmAttributes: None */
 #endif /* (SUPPORTED_FREQ_NBR > 1)  */
  0x00,                                 /* bLockDelayUnits: unit is milliseconds */
  0x00,                                 /* wLockDelay: 3 ms */
  0x00,
  /* 07 byte*/
  
 #if (AUDIO_SYNCHRO_FEEDBACK_ENABLED == 1)
  /* Feedback Endpoint - Standard Descriptor */
  0x09,                                 /* bLength */
  USB_ENDPOINT_DESCRIPTOR_TYPE,         /* bDescriptorType */
  AUDIO_FB_EP,                          /* bEndpointAddress */
  0x11,                                 /* bmAttributes: Isochrounous | Feedback */ 
  0x03,                                 
  0x00,                                 /* wMaxPacketSize in Bytes: 3 in FS mode, 4 in HS mode */
  0x01,                                 /* bInterval */
  FEEDBACK_REFRESH,                     /* bRefresh: 2^FEEDBACK_REFRESH ms */
  0x00                                  /* bSynchAddress */
  /* 09 byte*/
 #endif /* AUDIO_SYNCHRO_FEEDBACK_ENABLED */

/*--------------------------------------------------------------------------------------------------------------------*/
/*                                          USB Audio V2.0 descriptor                                                 */
/*             Not yet tested !                                                                                       */
/*--------------------------------------------------------------------------------------------------------------------*/
#elif defined(USB_AUDIO_V2_0)
  /* Configuration 1 */
  0x09,                                 /* bLength */
  USB_CONFIGURATION_DESCRIPTOR_TYPE,    /* bDescriptorType */
  LOBYTE(AUDIO_CONFIG_DESC_SIZE),       /* wTotalLength */
  HIBYTE(AUDIO_CONFIG_DESC_SIZE),      
  0x02,                                 /* bNumInterfaces */
  0x01,                                 /* bConfigurationValue */
  0x00,                                 /* iConfiguration */
  0xC0,                                 /* bmAttributes  BUS Powred*/
  0x32,                                 /* bMaxPower = 100 mA*/
  /* 09 byte*/

  /* USB Speaker Standard Interface Association Descriptor */
  0x08,                                 /* bLength */
  USB_INTERFACE_ASSOCIATION_TYPE,       /* bDescriptorType */
  0x00,                                 /* bFirstInterface: the first interface associated with this function */
  0x02,                                 /* bInterfaceCount: Number of contiguous interfaces 
                                           that are associated with this function */
  USB_DEVICE_CLASS_AUDIO,               /* bFunctionClass: Class Audio 0x01 */
  0x00,                                 /* bFunctionSubClass: not used */
  0x20,                                 /* bFunctionProtocol: USB Audio 2.0 */
  0x00,                                 /* iFunction */
  /* 08 byte*/

  /*----------------------------------------- 
                         USB Speaker Audio Control Interface 
                                             -------------------------------------------------*/
  /* USB Speaker Standard AC Interface Descriptor */
  AUDIO_INTERFACE_DESC_SIZE,            /* bLength */
  USB_INTERFACE_DESCRIPTOR_TYPE,        /* bDescriptorType */
  0x00,                                 /* bInterfaceNumber */
  0x00,                                 /* bAlternateSetting */
  0x00,                                 /* bNumEndpoints */
  USB_DEVICE_CLASS_AUDIO,               /* bInterfaceClass */
  AUDIO_SUBCLASS_AUDIOCONTROL,          /* bInterfaceSubClass */
  0x20,                                 /* bInterfaceProtocol */
  0x00,                                 /* iInterface */
  /* 09 byte*/  

  /* USB Speaker Class-specific AC Interface Descriptor */
  AUDIO_INTERFACE_DESC_SIZE,            /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_CONTROL_HEADER,                 /* bDescriptorSubtype */
  0x00,          /* 2.00 */             /* bcdADC */
  0x02,
  AUDIO_CATEGORY_DESKTOP_SPEAKER,       /* bCategory */
  0x3C,                                 /* wTotalLength total size of this descriptor 
                                           and all Clock, Unit and Terminal descriptors 
                                           = 9 + 8 + 17 + 14 + 12 */
  0x00,
  0x00,                                 /* bmControls: D1..0: Latency Control */
  /* 09 byte*/

  /* USB Speaker Clock Descriptor */
  0x08,                                 /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  0x0A,                                 /* bDescriptorSubtype: CLOCK_SOURCE */
  OUT_CLOCK_ID,                         /* bClockID */
  0x01,                                 /* bmAttributes: Internal fixed Clock */
  0x01,                                 /* bmControls: Clock Frequency Control (r) */
  0x00,                                 /* bAssocTerminal: None */
  0x00,                                 /* iClockSource */
  /* 08 byte*/  

  /* USB Speaker Input Terminal Descriptor */
  0x11,                                 /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_CONTROL_INPUT_TERMINAL,         /* bDescriptorSubtype */
  IN_TERMINAL_ID,                       /* bTerminal ID */
  0x01,                                 /* wTerminalType AUDIO_TERMINAL_USB_STREAMING   0x0101 */
  0x01,
  0x00,                                 /* bAssocTerminal: No association */
  OUT_CLOCK_ID,                         /* bCSourceID */
  0x01,                                 /* bNrChannels */
  0x01,                                 /* bmChannelConfig: Left Front */
  0x00,
  0x00,
  0x00,                                 
  0x00,                                 /* iChannelNames */
  0x00,                                 /* bmControls: No controls supported */
  0x00,                                 
  0x00,                                 /* iTerminal */
  /* 17 byte*/

  /* USB Speaker Audio Feature Unit Descriptor */
  0x0E,                                 /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_CONTROL_FEATURE_UNIT,           /* bDescriptorSubtype */
  OUT_FU_ID,                            /* bUnitID */
  IN_TERMINAL_ID,                       /* bSourceID */
  0x01,                                 /* bmaControls: Mute control (r) */ ///modify to 0x03 (rw)
  0x00,                                 
  0x00,                                 
  0x00,                                 
  0x00,                                 /* bmaControls: None */
  0x00,                                 
  0x00,                                 
  0x00,                                 
  0x00,                                 /* iFeature */
  /* 15 byte*/
  
  /*USB Speaker Output Terminal Descriptor */
  0x0C,                                 /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_CONTROL_OUTPUT_TERMINAL,        /* bDescriptorSubtype */
  OUT_TERMINAL_ID,                      /* bTerminalID */
  0x01,                                 /* wTerminalType  0x0301; Generic Speaker */
  0x03,
  0x00,                                 /* bAssocTerminal: No association */
  OUT_FU_ID,                            /* bSourceID */
  OUT_CLOCK_ID,                         /* bCSourceID */
  0x00,                                 /* bmControls: No controls supported */
  0x00,                                 
  0x00,                                 /* iTerminal */
  /* 12 byte*/


  /*----------------------------------------- 
                         USB Speaker Audio Streaming Interface 
                                             -------------------------------------------------*/

  /* USB Speaker Standard AS Interface Descriptor - Audio Streaming Zero Bandwith */
  /* Interface 1, Alternate Setting 0                                             */
  AUDIO_INTERFACE_DESC_SIZE,            /* bLength */
  USB_INTERFACE_DESCRIPTOR_TYPE,        /* bDescriptorType */
  0x01,                                 /* bInterfaceNumber */
  0x00,                                 /* bAlternateSetting */
  0x00,                                 /* bNumEndpoints */
  USB_DEVICE_CLASS_AUDIO,               /* bInterfaceClass */
  AUDIO_SUBCLASS_AUDIOSTREAMING,        /* bInterfaceSubClass */
  0x20,                                 /* bInterfaceProtocol */
  0x00,                                 /* iInterface */
  /* 09 byte*/

  /* USB Speaker Standard AS Interface Descriptor - Audio Streaming Operational */
  /* Interface 1, Alternate Setting 1                                           */
  AUDIO_INTERFACE_DESC_SIZE,            /* bLength */
  USB_INTERFACE_DESCRIPTOR_TYPE,        /* bDescriptorType */
  0x01,                                 /* bInterfaceNumber */
  0x01,                                 /* bAlternateSetting */
  #ifndef AUDIO_SYNCHRO_FEEDBACK_ENABLED  
  0x01,                                 /* bNumEndpoints */
  #else
  0x02,                                 /* bNumEndpoints */
  #endif /* AUDIO_SYNCHRO_FEEDBACK_ENABLED */
  USB_DEVICE_CLASS_AUDIO,               /* bInterfaceClass */
  AUDIO_SUBCLASS_AUDIOSTREAMING,        /* bInterfaceSubClass */
  0x20,                                 /* bInterfaceProtocol */
  0x00,                                 /* iInterface */
  /* 09 byte*/

  /* USB Speaker Class-Specific AS Interface Descriptor */
  0x10,                                 /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_STREAMING_GENERAL,              /* bDescriptorSubtype */
  IN_TERMINAL_ID,                       /* bTerminalLink */ 
  0x00,                                 /* bmControls: None */
  0x01,                                 /* bFormatType: FORMAT_TYPE_I */
  0x01,                                 /* bmFormats: PCM */
  0x00,
  0X00,
  0x00,                                 
  0x01,                                 /* bNrChannels */
  0x01,                                 /* bmChannelConfig: Spatial location Front Left */
  0x00,
  0x00,
  0x00,
  0x00,                                 /* iChannelNames */
  /* 16 byte*/

  /* USB Speaker Class-Specific AS Format Type Descriptor */
  0x06,                                 /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_STREAMING_FORMAT_TYPE,          /* bDescriptorSubtype */
  AUDIO_FORMAT_TYPE_I,                  /* bFormatType */   
  0x02,                                 /* bSubslotSize: 16-bit */
  0x10,                                 /* bBitResolution (16-bits per sample) */   
  /* 6 bytes */
  
  /* USB Speaker Standard AS Isochronous Data Endpoint Descriptor */
  0x07,                                 /* bLength */
  USB_ENDPOINT_DESCRIPTOR_TYPE,         /* bDescriptorType */
  AUDIO_OUT_EP,                         /* bEndpointAddress out endpoint*/
  0x05,                                 /* bmAttributes: Isochrnous | Asynchronous | Data Endpoint */ 
  0xFF,
  0x03,                                 /* wMaxPacketSize: 1023 bytes */
  0x01,                                 /* bInterval must be 1*/
  /* 07 byte*/
  
  /* USB Speaker Class-Specific AS Isochronous Data Endpoint Descriptor */
  0x08,                                 /* bLength */
  AUDIO_ENDPOINT_DESCRIPTOR_TYPE,       /* bDescriptorType */
  AUDIO_ENDPOINT_GENERAL,               /* bDescriptor */
  0x00,                                 /* bmAttributes */
  0x00,                                 /* bmControls */
  0x00,                                 /* bLockDelayUnits */
  0x00,                                 /* wLockDelay */
  0x00
  /* 08 byte*/
  
 #ifdef AUDIO_SYNCHRO_FEEDBACK_ENABLED
  /* Feedback Endpoint - Standard Descriptor */
  ,0x07,                                /* bLength */
  USB_ENDPOINT_DESCRIPTOR_TYPE,         /* bDescriptorType */
  AUDIO_FB_EP,                          /* bEndpointAddress */
  0x11,                                 /* bmAttributes: Isochrounous | Feedback */ 
  0x04,                                 
  0x00,                                 /* wMaxPacketSize in Bytes: 3 in FS mode, 4 in HS mode */
  0x01                                  /* bInterval */
  /* 07 byte*/
 #endif /* AUDIO_SYNCHRO_FEEDBACK_ENABLED */

#endif /* USB_AUDIO_V1_0 */
};
#endif /* USE_USB_OTG_HS */



/* AUDIO interface class callbacks structure */
USBD_Class_cb_TypeDef  AUDIO_cb = 
{
  usbd_audio_Init,
  usbd_audio_DeInit,
  usbd_audio_Setup,
  NULL, /* EP0_TxSent */
  usbd_audio_EP0_RxReady,
  usbd_audio_DataIn,
  usbd_audio_DataOut,
  usbd_audio_SOF,
  usbd_audio_IN_Incplt,
  usbd_audio_OUT_Incplt,
  USBD_audio_GetCfgDesc,
#ifdef USB_OTG_HS_CORE  
  USBD_audio_GetCfgDesc, /* use same config as per FS */
#endif 
};


/**
  * @}
  */ 

/** @defgroup usbd_audio_Private_Functions
  * @{
  */ 

/**
* @brief  usbd_audio_Init
*         Initilaize the AUDIO interface
* @param  pdev: device instance
* @param  cfgidx: Configuration index
* @retval status
*/
static uint8_t  usbd_audio_Init (void  *pdev, 
                                 uint8_t cfgidx)
{  
  /* Open EP OUT */
  DCD_EP_Open(pdev,
              AUDIO_OUT_EP,
              AUDIO_OUT_MPS,
              USB_OTG_EP_ISOC);
  
#if (AUDIO_IN_ENABLED == 1) 
  /* Open EP IN */
  DCD_EP_Open(pdev,
              AUDIO_IN_EP,
              AUDIO_IN_MPS,
              USB_OTG_EP_ISOC);
  
  /* Initialize the Audio output Hardware layer */
  if (AUDIO_IN_fops.Init(DEFAULT_IN_AUDIO_FREQ, DEFAULT_VOLUME, 0) != USBD_OK)
  {
    return USBD_FAIL;
  }

  /* Initialize the IN variables */
  IsocInWrPtr = IsocInBuff;
  IsocInRdPtr = IsocInBuff;
  IsocInWrBufDescIdx = 0;
  IsocInRdBufDescIdx = 0;
  IsocInWrState = STATE_IDLE;
  IsocInRdState = STATE_IDLE;
  InEmptyFrameCount = 0;  
  
  /* Flush endpoint */
  DCD_EP_Flush(pdev, AUDIO_IN_EP); 
  
  /* Prepare next buffer to be sent: dummy data */
  DCD_EP_Tx (pdev,
             AUDIO_IN_EP,
             IsocInBuffDummy,
             IsocInCurPacketSze);
  
#endif /* AUDIO_IN_ENABLED */
  
  /* Initialize the Audio output Hardware layer */
  if (AUDIO_OUT_fops.Init(DEFAULT_OUT_AUDIO_FREQ, DEFAULT_VOLUME, 0) != USBD_OK)
  {
    return USBD_FAIL;
  }
  
  /* Initialize the audio correction mechanism */
  if (AUDIO_CORR_Init(DEFAULT_OUT_AUDIO_FREQ) != 0)
  {
    return USBD_FAIL;
  }

#if (AUDIO_SYNCHRO_FEEDBACK_ENABLED == 1)
  if (AUDIO_OUT_fops.pOptions[0] == SYNCH_FEEDBACK)
  {    
    /* Open Feedback */
    DCD_EP_Open(pdev,
                AUDIO_FB_EP,
                FB_PCKT_SZE,
                USB_OTG_EP_ISOC);
    
    /*flush endpoint*/
    DCD_EP_Flush(pdev, AUDIO_FB_EP);
    
    /* Prepare next data to be sent */
    DCD_EP_Tx (pdev,
               AUDIO_FB_EP,
               pFBBuf,
               FB_PCKT_SZE);
  }
#endif /* AUDIO_SYNCHRO_FEEDBACK_ENABLED */
  
  /* Initialize the first element of the linked chain */
  IsocOutBufDesc[0].Size = IsocOutCurPacketSze;
  IsocOutBufDesc[0].Next = IsocOutBuff + IsocOutCurPacketSze;
  //IsocOutBufDesc[0].Next = IsocOutBuff + IsocOutPacketSze + OUT_BUF_MARGIN;
  IsocOutWrPtr = IsocOutBuff;  
  IsocOutWrBufDescIdx = 0;
  OutEmptyFrameCount = 0;  
  IsocOutWrState = STATE_IDLE;
  
  /* Reset Read parameters */
  IsocOutRdPtr = IsocOutBuff;
  IsocOutRdBufDescIdx = 0;
  IsocOutRdState = STATE_IDLE;
  
  /* Prepare Out endpoint to receive audio data */
  DCD_EP_PrepareRx(pdev,
                   AUDIO_OUT_EP,
                   IsocOutBuff,
                   IsocOutCurPacketSze);

#ifdef AUDIO_SYNCHRO_SRC_ENABLED
  //STM32_Resample_Init(0, 0, 0, 0);
#endif
  
  
#ifdef AUDIO_SYNCHRO_SRC_ENABLED
  ratio = RATIO_1;
#endif /* AUDIO_SYNCHRO_SRC_ENABLED */  

  return USBD_OK;
}

/**
* @brief  usbd_audio_DeInit
*         DeInitialize the AUDIO layer
* @param  pdev: device instance
* @param  cfgidx: Configuration index
* @retval status
*/
static uint8_t  usbd_audio_DeInit (void  *pdev, 
                                   uint8_t cfgidx)
{ 
  /* Close all used endpoints */
  DCD_EP_Close (pdev , AUDIO_OUT_EP);
  DCD_EP_Close (pdev , AUDIO_FB_EP);
  
  /* DeInitialize the Audio output Hardware layer */
  if (AUDIO_OUT_fops.DeInit(0) != USBD_OK)
  {
    return USBD_FAIL;
  }
  
  return USBD_OK;
}

/**
  * @brief  usbd_audio_Setup
  *         Handle the AUDIO specific requests
  * @param  pdev: instance
  * @param  req: usb requests
  * @retval status
  */
static uint8_t  usbd_audio_Setup (void  *pdev, 
                                  USB_SETUP_REQ *req)
{
  uint16_t len;
  uint8_t  *pbuf;

  switch (req->bmRequest & USB_REQ_TYPE_MASK)
  {
    /* AUDIO Class Requests -------------------------------*/
  case USB_REQ_TYPE_CLASS :
#ifdef USB_AUDIO_V1_0
    switch (req->bRequest)
    {
    case AUDIO_REQ_GET_CUR:
      AUDIO_Req_GetCurrent(pdev, req);
      break;
      
    case AUDIO_REQ_SET_CUR:
      AUDIO_Req_SetCurrent(pdev, req);   
      break;

    default:
      USBD_CtlError (pdev, req);
      return USBD_FAIL;
    }
    break;
#elif defined (USB_AUDIO_V2_0)
    switch (req->bRequest)
    {
      case AUDIO_REQ_CUR:
        /* Get Request */
        if (req->bmRequest & 0x80)
        {
          /* Call Class-Specific Get Request function */
          AUDIO_Req_Get(pdev, req);
        }
        else /* Set Request */
        {
          /* Call Class-Specific Set Request function */
          AUDIO_Req_Set(pdev, req);
        }
        break;
      
      case AUDIO_REQ_RANGE:
        /* Get Request */
        if (req->bmRequest & 0x80)
        {
          /* Call Class-Specific Get Request function */
          AUDIO_Req_Get(pdev, req);
        }
        else /* Set Request */
        {
          /* Call Class-Specific Set Request function */
          AUDIO_Req_Set(pdev, req);
        }
        break;

      case AUDIO_REQ_MEM: /* Not supported */
        USBD_CtlError (pdev, req);
        return USBD_FAIL;      
      
      default:
        USBD_CtlError (pdev, req);
        return USBD_FAIL;
    }
    break;
#else
 #error "ERROR: Missing define USB_AUDIO_V1_0 or USB_AUDIO_V2_0 in usbd_audio_core.h"
#endif /* USB_AUDIO_V1_0 */
    
    /* Standard Requests -------------------------------*/
  case USB_REQ_TYPE_STANDARD:
    switch (req->bRequest)
    {
    case USB_REQ_GET_DESCRIPTOR: 
      if( (req->wValue >> 8) == AUDIO_DESCRIPTOR_TYPE)
      {
#ifdef USB_OTG_HS_INTERNAL_DMA_ENABLED
        pbuf = usbd_audio_Desc;   
#else
        pbuf = usbd_audio_CfgDesc + 18;
#endif 
        len = MIN(USB_AUDIO_DESC_SIZ , req->wLength);
      }
      
      USBD_CtlSendData (pdev, 
                        pbuf,
                        len);
      break;
      
    case USB_REQ_GET_INTERFACE :
      USBD_CtlSendData (pdev,
                        (uint8_t *)&usbd_audio_AltSet,
                        1);
      break;
      
    case USB_REQ_SET_INTERFACE :
      if ((uint8_t)(req->wValue) < AUDIO_TOTAL_IF_NUM)
      {
        usbd_audio_AltSet = (uint8_t)(req->wValue);
      }
      else
      {
        /* Call the error management function (command will be nacked */
        USBD_CtlError (pdev, req);
      }
      break;
    }
  }
  return USBD_OK;
}

/**
  * @brief  usbd_audio_EP0_RxReady
  *         Data received on control endpoint
  * @param  pdev: device device instance
  * @retval status
  */
static uint8_t  usbd_audio_EP0_RxReady (void  *pdev)
{ 
#if defined(USB_AUDIO_V1_0)
  /* Check if an AudioControl request has been issued */
  if (AudioCtlCmd == AUDIO_REQ_SET_CUR)
  {/* In this driver, to simplify code, only SET_CUR request is managed */
    /* Check for which addressed unit the AudioControl request has been issued */
    if (AudioCtlUnit == AUDIO_OUT_STREAMING_CTRL)
    {/* In this driver, to simplify code, only one unit is manage */
      /* Call the audio interface mute function */
      AUDIO_OUT_fops.MuteCtl(AudioCtl[0]);
      
      /* Reset the AudioCtlCmd variable to prevent re-entering this function */
      AudioCtlCmd = 0;
      AudioCtlLen = 0;
      AudioCtlUnit = 0;
      AudioTrgtEpt = 0;
    }
    /* Check which endpoint is addressed by the request */
    else if (AudioTrgtEpt == 0x01)
    {      
  #if (SUPPORTED_FREQ_NBR > 1)
      /* Call the audio interface control */
      IsocOutCurSampleFreq = AUDIO_OUT_fops.OptionCtrl(AudioCtl, AudioCtlLen);
      
      /* Update the current parameters */
      IsocOutPacketSze  = (IsocOutCurSampleFreq * (DEFAULT_OUT_BIT_RESOLUTION/8) * 2) / 1000;
      IsocOutCurPacketSze  = (IsocOutCurSampleFreq * (DEFAULT_OUT_BIT_RESOLUTION/8) * 2) / 1000; ///Is this step necessary??
      
      /// Reset all parameters to default while synchronization is ongoing !!!??
      /// if host requests audio freq change while file is playing ? To be verified.
    #if (AUDIO_SYNCHRO_FEEDBACK_ENABLED == 1)
      /* Current samples number */
      IsocOutSampleNbr = (IsocOutCurSampleFreq/1000);
      IsocOutSampleFrct = IsocOutCurSampleFreq - ((IsocOutCurSampleFreq/1000) * 1000);      
      IsocOutCurSampleNbrTb[0] = (IsocOutCurSampleFreq/1000);      
      IsocOutCurSampleNbrTb[1] = IsocOutCurSampleFreq - ((IsocOutCurSampleFreq/1000) * 1000);
      IsocOutCurSampleNbrTb[2] = (IsocOutCurSampleFreq/1000);
      IsocOutCurSampleNbrTb[3] = IsocOutCurSampleFreq - ((IsocOutCurSampleFreq/1000) * 1000);
      
      
      /* Update Feedback information */
      pFBBuf[0] = FEEDBACK_FF_FS_B2(IsocOutCurSampleNbrTb[0], IsocOutCurSampleNbrTb[1]);
      pFBBuf[1] = FEEDBACK_FF_FS_B1(IsocOutCurSampleNbrTb[0], IsocOutCurSampleNbrTb[1]);
      pFBBuf[2] = FEEDBACK_FF_FS_B0(IsocOutCurSampleNbrTb[0], IsocOutCurSampleNbrTb[1]);
     #endif /* AUDIO_SYNCHRO_FEEDBACK_ENABLED */
    #endif /* SUPPORTED_FREQ_NBR > 1 */
      
      /* Reset the AudioCtlCmd variable to prevent re-entering this function */
      AudioCtlCmd = 0;
      AudioCtlLen = 0; 
      AudioCtlUnit = 0;
      AudioTrgtEpt = 0;
      
      ///TO DO:
      ///Should Stall the endpoint 0 if the requested audio frequency is not supported!
    }
  } 
#elif defined(USB_AUDIO_V2_0)
  /* Check if an AudioControl request has been issued */
  if (AudioCtlCmd == AUDIO_REQ_CUR)
  {
    /* Check for which addressed unit the AudioControl request has been issued */
    switch (AudioCtlUnit)
    {
      case OUT_FU_ID:
        /* Call the audio interface mute function */
        AUDIO_OUT_fops.MuteCtl(AudioCtl[0]);
        MuteCtrl = AudioCtl[0];
        break;
      
      case OUT_CLOCK_ID:
    #if (SUPPORTED_FREQ_NBR == 1)
        /* Not supported */
    #else
      #error "ERROR: Multiple sampling frequecies not yet supported!"      
    #endif /* SUPPORTED_FREQ_NBR */
        break;
      
      default:
        /* Call the error management function (command will be stalled */
        ///USBD_CtlError (pdev, req);
        break;
    }
    
    ///    /* Check which endpoint is addressed by the request */
    //    if (AudioTrgtEpt == 0x01)
    //    {
    //      /* Not supported */
    //    }
    
    /* Reset the AudioCtlCmd variable to prevent re-entering this function */
    AudioCtlCmd = 0;
    AudioCtlLen = 0;
    AudioCtlUnit = 0;
    AudioTrgtEpt = 0;    
  } 
#else
 #error "ERROR: Missing define of USB_AUDIO_V1_0 or USB_AUDIO_V2_0 in usbd_audio_core.h!"
#endif /* USB_AUDIO_V2_0 */
  
  return USBD_OK;
}

/**
  * @brief  usbd_audio_DataIn
  *         
  * @param  pdev: instance
  * @param  epnum: endpoint number
  * @retval status
  */
static uint8_t  usbd_audio_DataIn (void *pdev, uint8_t epnum)
{
#if (AUDIO_SYNCHRO_FEEDBACK_ENABLED == 1)  
  if (epnum == (AUDIO_FB_EP & 0x7F))
  {
    if (((USB_OTG_CORE_HANDLE*)pdev)->dev.device_status == USB_OTG_CONFIGURED )
    {
      /* Flush endpoint */
      DCD_EP_Flush(pdev, AUDIO_FB_EP);
      
      /* Prepare next data to be sent */
      DCD_EP_Tx (pdev,
                 AUDIO_FB_EP,
                 pFBBuf,
                 FB_PCKT_SZE);      
    }
  }
#endif /* AUDIO_SYNCHRO_FEEDBACK_ENABLED */
  
#if (AUDIO_IN_ENABLED == 1) 
  if (epnum == (AUDIO_IN_EP & 0x7F))
  {
    /* Flush endpoint */
    DCD_EP_Flush(pdev, AUDIO_IN_EP); ///Can be removed ??
      
    if ((IsocInWrState & STATE_RUN) || (IsocInWrState == STATE_READY))
    {        
      /* Toggle the frame index */  
      ((USB_OTG_CORE_HANDLE*)pdev)->dev.in_ep[epnum].even_odd_frame = 
        (((USB_OTG_CORE_HANDLE*)pdev)->dev.in_ep[epnum].even_odd_frame)? 0:1;
        
      if (IsocInRdBufDescIdx == 0)
      {        
        /* Prepare next buffer to be sent */
        DCD_EP_Tx (pdev,
                   AUDIO_IN_EP,
                   IsocInBuff,
                   IsocInBufDesc[0].Size);
        
        /* Set the current read pointer */
        IsocInRdPtr = IsocInBufDesc[0].Next;
      }
      else
      {          
        /* Prepare next buffer to be sent */
        DCD_EP_Tx (pdev,
                   AUDIO_IN_EP,
                   IsocInBufDesc[IsocInRdBufDescIdx - 1].Next,
                   IsocInBufDesc[IsocInRdBufDescIdx].Size);
        
        /* Set the current read pointer */
        IsocInRdPtr = IsocInBufDesc[IsocInRdBufDescIdx - 1].Next;
      }
      
      /* Increment the read buffer descriptor index */
      IsocInRdBufDescIdx++;  
      
      /* Check if the end of the the buffer has been reached */
      if (IsocInRdBufDescIdx >= IN_PACKET_NUM)
      {
        /* All buffers are full: roll back */
        IsocInRdBufDescIdx = 0;
      }
    }
    else if ((IsocInWrState == STATE_IDLE) || (IsocInWrState == STATE_BUFFERING))
    {      
      /* Start recording */
      if (IsocInWrState == STATE_IDLE) 
      {
        /* Update the current state: the audio recorder has to fill the buffer with data
        then when half of total buffers are full, the recorded data will be sent over USB.
        Meanwhile 0x00 data packets will be sent to host (meaning no recorded frame is present / silent). */
        IsocInWrState = STATE_BUFFERING;
        
        /* Start recording */
        AUDIO_IN_fops.AudioCmd(IsocInBuff,                                     /* Samples buffer pointer */
                               IsocInCurPacketSze,                             /* Number of samples in Bytes */
                               AUDIO_IN_CMD_START);                            /* Command to be processed */
      }

    /* Toggle the frame index */  
    ((USB_OTG_CORE_HANDLE*)pdev)->dev.in_ep[epnum].even_odd_frame = 
      (((USB_OTG_CORE_HANDLE*)pdev)->dev.in_ep[epnum].even_odd_frame)? 0:1;
      
      /* Prepare next buffer to be sent: dummy data */
      DCD_EP_Tx (pdev,
                 AUDIO_IN_EP,
                 IsocInBuffDummy,
                 IsocInCurPacketSze);
    }
    
    /* Decrement the empty frame counter */
    if (InEmptyFrameCount)
    {
      InEmptyFrameCount --;
    }  
  }
#endif /* AUDIO_IN_ENABLED */
  
  return USBD_OK;
}

///**
//  * @brief  usbd_audio_DataOut
//  *         
//  * @param  pdev: instance
//  * @param  epnum: endpoint number
//  * @retval status
//  */
//void * pDev;
//uint8_t epNum;
//
//static uint8_t  usbd_audio_DataOut (void *pdev, uint8_t epnum)
//{     
//#if (AUDIO_SYNCHRO_ADDREM_ENABLED == 1) 
//  uint8_t* ptr;
//#endif
//  
//  if (epnum == AUDIO_OUT_EP)
//  {  
//    /* Set the Isochrounous Buffer Out linked chain parameters */
//    IsocOutBufDesc[IsocOutWrBufDescIdx].Size = ((USB_OTG_CORE_HANDLE*)pdev)->dev.out_ep[epnum].xfer_count;///.xfer_len;
//    IsocOutBufDesc[IsocOutWrBufDescIdx].Next = (uint8_t*)(IsocOutWrPtr + IsocOutBufDesc[IsocOutWrBufDescIdx].Size);
//
//#if (AUDIO_SYNCHRO_SRC_ENABLED == 1)
//    if (AUDIO_OUT_fops.pOptions[0] == SYNCH_SRC)
//    {
//      /* Wait to make sure previous resampling is finished */
//      while (SRCProcessing != 0)
//      {}
//      
//      /* Perform resampling transformation (this function is using DMA) */    
//      STM32_Resample_Process((int*)IsocOutWrPtr,
//                             (IsocOutPacketSze/4), 
//                             (int*)IsocOutWrPtr,
//                             ratio,
//                             (uint32_t *)(&SrcOutSize));
//
//      /* Wait till current processing is done */
//      SRCProcessing = 1;
//      pDev = pdev;
//      epNum = epnum;
////      while (SRCProcessing != 0)
////      {}
//      
//      /* Make sure no excessive over/under-sampling occured */
////      if (SrcOutSize > 49)       SrcOutSize = 49;
////      if (SrcOutSize < 47)       SrcOutSize = 47;
//       
////      /* Update the Current buffer size and next buffer start address */
////      IsocOutBufDesc[IsocOutWrBufDescIdx].Size = (SrcOutSize * 4);
////      (IsocOutBufDesc[IsocOutWrBufDescIdx].Next) = (uint8_t*)(IsocOutWrPtr + IsocOutBufDesc[IsocOutWrBufDescIdx].Size);
////      
////      /* Reset the original data size */
////      IsocOutCurPacketSze = IsocOutPacketSze;
//    }
//    else
//    {
//#endif /* AUDIO_SYNCHRO_SRC_ENABLED */
//
//#if (AUDIO_SYNCHRO_ADDREM_ENABLED == 1)
//    if (AUDIO_OUT_fops.pOptions[0] == SYNCH_ADDREM)
//    {
//      /* Application has to add a sample */
//      if (IsocOutCurPacketSze > IsocOutPacketSze)
//      {
//        /* Add data samples */
//        IsocOutBufDesc[IsocOutWrBufDescIdx].Size += (uint32_t)(IsocOutCurPacketSze - IsocOutPacketSze);
//        (IsocOutBufDesc[IsocOutWrBufDescIdx].Next) += IsocOutCurPacketSze - IsocOutPacketSze;
//        
//        /* Padding of added bytes with average value */
//        /* Get the address of last item */
//        ptr = (uint8_t*)((IsocOutBufDesc[IsocOutWrBufDescIdx].Next) - 2);
//        *(uint16_t*)(ptr) = *(uint16_t*)((uint16_t*)ptr - 2);
//        *(uint16_t*)((uint16_t*)ptr - 1) = *(uint16_t*)(ptr);
//      }
//      /* Application has to remove a sample */
//      else if (IsocOutCurPacketSze < IsocOutPacketSze)
//      {
//        /* Remove data samples */
//        IsocOutBufDesc[IsocOutWrBufDescIdx].Size -= (uint32_t)(IsocOutPacketSze - IsocOutCurPacketSze);
//        (IsocOutBufDesc[IsocOutWrBufDescIdx].Next) -= IsocOutPacketSze - IsocOutCurPacketSze;
//      }
//      
//      /* Reset the original data size */
//      IsocOutCurPacketSze = IsocOutPacketSze;
//    }
//    else
//    {
//#else
//      /* Update the current size of packets */
//      if (IsocOutBufDesc[IsocOutWrBufDescIdx].Size != IsocOutPacketSze) ///This test is not needed, just for debug (keep only the packet size update)
//      {
//        IsocOutCurPacketSze = IsocOutBufDesc[IsocOutWrBufDescIdx].Size;
//        
//        ///
//        // STM_EVAL_LEDToggle(LED2); 
//      }
//#endif /* AUDIO_SYNCHRO_ADDREM_ENABLED */
//#if (AUDIO_SYNCHRO_ADDREM_ENABLED == 1)
//    }
//#endif
//   
//#ifdef  AUDIO_SYNCHRO_SRC_ENABLED
//    }
//#endif /* AUDIO_SYNCHRO_SRC_ENABLED */
//
//#if (AUDIO_SYNCHRO_SRC_ENABLED != 1)
//    /* Increment the write buffer pointer */
//    IsocOutWrPtr += IsocOutBufDesc[IsocOutWrBufDescIdx].Size;
//    //IsocOutWrPtr += IsocOutBufDesc[IsocOutWrBufDescIdx].Size + OUT_BUF_MARGIN;
//    
//    /* Increment the linked chain member index */
//    IsocOutWrBufDescIdx++;
//    
//    /* Check if the end of the the buffer has been reached */
//    if (IsocOutWrBufDescIdx >= OUT_PACKET_NUM)
//    {
//      /* All buffers are full: roll back */
//      IsocOutWrPtr = IsocOutBuff;
//      IsocOutBufDesc[IsocOutWrBufDescIdx - 1].Next = IsocOutBuff;
//      IsocOutWrBufDescIdx = 0;          
//    }
//    
//    /* Toggle the frame index */  
//    ((USB_OTG_CORE_HANDLE*)pdev)->dev.out_ep[epnum].even_odd_frame = 
//      (((USB_OTG_CORE_HANDLE*)pdev)->dev.out_ep[epnum].even_odd_frame)? 0:1;
//      
//    
//    /* Check and update the buffering state machine */
//    switch (IsocOutWrState)
//    {
//      case STATE_IDLE:
//        IsocOutWrState = STATE_BUFFERING;
//        break;
//        
//      case STATE_BUFFERING:
//        /* Check if the buffer index reached the half of total out buffer */
//        if (IsocOutWrBufDescIdx >= (OUT_PACKET_NUM / 2))
//        {
//          /* Switch current state to Ready: allow Read operation from the buffer */
//          IsocOutWrState = STATE_READY;
//        }
//        break;
//        
//      case STATE_READY:
//        /* Decrement the empty frame counter */
//        if (OutEmptyFrameCount)
//        {
//          OutEmptyFrameCount --;
//        }      
//        break;
//       
//      case STATE_CLOSING:        
//        break;
//        
//      default:
//        break;
//    }
//
//    /* Prepare Out endpoint to receive next audio packet */
//    DCD_EP_PrepareRx(pdev,
//                     AUDIO_OUT_EP,
//                     IsocOutWrPtr,
//                     IsocOutCurPacketSze);///IsocOutPacketSze);
//#endif /* AUDIO_SYNCHRO_SRC_ENABLED */    
//  }
//
//
//  return USBD_OK;
//}

/**
* @brief  usbd_audio_DataOut
*         
* @param  pdev: instance
* @param  epnum: endpoint number
* @retval status
*/
void * pDev;
uint8_t epNum;
static uint8_t  usbd_audio_DataOut (void *pdev, uint8_t epnum)
{     
  uint8_t* ptr;
  
  if (epnum == AUDIO_OUT_EP)
  {  
    /* Set the Isochrounous Buffer Out linked chain parameters */
    IsocOutBufDesc[IsocOutWrBufDescIdx].Size = ((USB_OTG_CORE_HANDLE*)pdev)->dev.out_ep[epnum].xfer_count;///.xfer_len;
    IsocOutBufDesc[IsocOutWrBufDescIdx].Next = (uint8_t*)(IsocOutWrPtr + IsocOutBufDesc[IsocOutWrBufDescIdx].Size);

#if (AUDIO_SYNCHRO_SRC_ENABLED != 0)
    if (AUDIO_OUT_fops.pOptions[0] == SYNCH_SRC)
    {
      /* Wait to make sure previous resampling is finished */
      //while (SRCProcessing != 0)
      //{}
      
      /* Perform resampling transformation (this function is using DMA) */    
      STM32_Resample_Process((int*)IsocOutWrPtr,
                             (IsocOutPacketSze), 
                             (int*)IsocOutWrPtr,
                             ratio,
                             (uint32_t *)(&SrcOutSize));
      
      /* Wait till current processing is done */
      //SRCProcessing = 1;
      pDev = pdev;
      epNum = epnum;
      //      while (SRCProcessing != 0)
      //      {}
      
      /* Make sure no excessive over/under-sampling occured */
      //      if (SrcOutSize > 49)       SrcOutSize = 49;
      //      if (SrcOutSize < 47)       SrcOutSize = 47;
      
      //      /* Update the Current buffer size and next buffer start address */
      //      IsocOutBufDesc[IsocOutWrBufDescIdx].Size = (SrcOutSize * 4);
      //      (IsocOutBufDesc[IsocOutWrBufDescIdx].Next) = (uint8_t*)(IsocOutWrPtr + IsocOutBufDesc[IsocOutWrBufDescIdx].Size);
      //      
      //      /* Reset the original data size */
      //      IsocOutCurPacketSze = IsocOutPacketSze;
    }
#else
    if (0)
    {
    }
#endif /* AUDIO_SYNCHRO_SRC_ENABLED */
    else
    {
      if (AUDIO_OUT_fops.pOptions[0] == SYNCH_ADDREM)
      {
        /* Application has to add a sample */
        if (IsocOutCurPacketSze > IsocOutPacketSze)
        {
          /* Add data samples */
          IsocOutBufDesc[IsocOutWrBufDescIdx].Size += (uint32_t)(IsocOutCurPacketSze - IsocOutPacketSze);
          (IsocOutBufDesc[IsocOutWrBufDescIdx].Next) += IsocOutCurPacketSze - IsocOutPacketSze;
          
          /* Padding of added bytes with average value */
          /* Get the address of last item */
          ptr = (uint8_t*)((IsocOutBufDesc[IsocOutWrBufDescIdx].Next) - 2);
          *(uint16_t*)(ptr) = *(uint16_t*)((uint16_t*)ptr - 2);
          *(uint16_t*)((uint16_t*)ptr - 1) = *(uint16_t*)(ptr);
        }
        /* Application has to remove a sample */
        else if (IsocOutCurPacketSze < IsocOutPacketSze)
        {
          /* Remove data samples */
          IsocOutBufDesc[IsocOutWrBufDescIdx].Size -= (uint32_t)(IsocOutPacketSze - IsocOutCurPacketSze);
          (IsocOutBufDesc[IsocOutWrBufDescIdx].Next) -= IsocOutPacketSze - IsocOutCurPacketSze;
        }
        
        /* Reset the original data size */
        IsocOutCurPacketSze = IsocOutPacketSze;
      }
      else
      {
        /* Update the current size of packets */
        if (IsocOutBufDesc[IsocOutWrBufDescIdx].Size != IsocOutPacketSze) ///This test is not needed, just for debug (keep only the packet size update)
        {
          IsocOutCurPacketSze = IsocOutBufDesc[IsocOutWrBufDescIdx].Size;
          
          ///
          // STM_EVAL_LEDToggle(LED2); 
        }
      }
    }

#if (AUDIO_SYNCHRO_SRC_ENABLED != 0)
    if (AUDIO_OUT_fops.pOptions[0] != SYNCH_SRC)
#endif /* AUDIO_SYNCHRO_SRC_ENABLED */
    {
      /* Increment the write buffer pointer */
      IsocOutWrPtr += IsocOutBufDesc[IsocOutWrBufDescIdx].Size;
      //IsocOutWrPtr += IsocOutBufDesc[IsocOutWrBufDescIdx].Size + OUT_BUF_MARGIN;
      
      /* Increment the linked chain member index */
      IsocOutWrBufDescIdx++;
      
      /* Check if the end of the the buffer has been reached */
      if (IsocOutWrBufDescIdx >= OUT_PACKET_NUM)
      {
        /* All buffers are full: roll back */
        IsocOutWrPtr = IsocOutBuff;
        IsocOutBufDesc[IsocOutWrBufDescIdx - 1].Next = IsocOutBuff;
        IsocOutWrBufDescIdx = 0;          
      }
      
      /* Toggle the frame index */  
      ((USB_OTG_CORE_HANDLE*)pdev)->dev.out_ep[epnum].even_odd_frame = 
        (((USB_OTG_CORE_HANDLE*)pdev)->dev.out_ep[epnum].even_odd_frame)? 0:1;
        
        
        /* Check and update the buffering state machine */
        switch (IsocOutWrState)
        {
        case STATE_IDLE:
          IsocOutWrState = STATE_BUFFERING;
          break;
          
        case STATE_BUFFERING:
          /* Check if the buffer index reached the half of total out buffer */
          if (IsocOutWrBufDescIdx >= (OUT_PACKET_NUM / 2))
          {
            /* Switch current state to Ready: allow Read operation from the buffer */
            IsocOutWrState = STATE_READY;
          }
          break;
          
        case STATE_READY:
          /* Decrement the empty frame counter */
          if (OutEmptyFrameCount)
          {
            OutEmptyFrameCount --;
          }      
          break;
          
        case STATE_CLOSING:        
          break;
          
        default:
          break;
        }
        
        /* Prepare Out endpoint to receive next audio packet */
        DCD_EP_PrepareRx(pdev,
                         AUDIO_OUT_EP,
                         IsocOutWrPtr,
                         IsocOutCurPacketSze);///IsocOutPacketSze);
    }
  }
  
  return USBD_OK;
}

#if (AUDIO_SYNCHRO_SRC_ENABLED == 1)
/**
  * @brief  SRC_ProcessCompleteCallback
  *         
  * @param  None
  * @retval None
  */
void STM32_Resample_ProcessCpltCallback(void)
//void SRC_ProcessCompleteCallback(void)
{
  /* Update the Current buffer size and next buffer start address */
  IsocOutBufDesc[IsocOutWrBufDescIdx].Size = (SrcOutSize * 4);
  (IsocOutBufDesc[IsocOutWrBufDescIdx].Next) = (uint8_t*)(IsocOutWrPtr + IsocOutBufDesc[IsocOutWrBufDescIdx].Size);
  
  /* Reset the original data size */
  IsocOutCurPacketSze = IsocOutPacketSze;
  
  /* Increment the write buffer pointer */
  IsocOutWrPtr += IsocOutBufDesc[IsocOutWrBufDescIdx].Size;
  //IsocOutWrPtr += IsocOutBufDesc[IsocOutWrBufDescIdx].Size + OUT_BUF_MARGIN;
  
  /* Increment the linked chain member index */
  IsocOutWrBufDescIdx++;
  
  /* Check if the end of the the buffer has been reached */
  if (IsocOutWrBufDescIdx >= OUT_PACKET_NUM)
  {
    /* All buffers are full: roll back */
    IsocOutWrPtr = IsocOutBuff;
    IsocOutBufDesc[IsocOutWrBufDescIdx - 1].Next = IsocOutBuff;
    IsocOutWrBufDescIdx = 0;          
  }
  
  /* Toggle the frame index */  
  ((USB_OTG_CORE_HANDLE*)pDev)->dev.out_ep[epNum].even_odd_frame = 
    (((USB_OTG_CORE_HANDLE*)pDev)->dev.out_ep[epNum].even_odd_frame)? 0:1;
    
  
  /* Check and update the buffering state machine */
  switch (IsocOutWrState)
  {
    case STATE_IDLE:
      IsocOutWrState = STATE_BUFFERING;
      break;
      
    case STATE_BUFFERING:
      /* Check if the buffer index reached the half of total out buffer */
      if (IsocOutWrBufDescIdx >= (OUT_PACKET_NUM / 2))
      {
        /* Switch current state to Ready: allow Read operation from the buffer */
        IsocOutWrState = STATE_READY;
      }
      break;
      
    case STATE_READY:
      /* Decrement the empty frame counter */
      if (OutEmptyFrameCount)
      {
        OutEmptyFrameCount --;
      }      
      break;
     
    case STATE_CLOSING:        
      break;
      
    default:
      break;
  }

  /* Prepare Out endpoint to receive next audio packet */
  DCD_EP_PrepareRx(pDev,
                   AUDIO_OUT_EP,
                   IsocOutWrPtr,
                   IsocOutCurPacketSze);///IsocOutPacketSze);
}
#endif /* AUDIO_SYNCHRO_SRC_ENABLED */

/**
  * @brief  usbd_audio_BuffXferCplt
  *         Manage end of buffer transfer for each device.
  * @param  Direction: could be DIRECTION_IN or DIRECTION_OUT
  * @param  pbuf: Pointer to the address of the current buffer
  * @param  pSize: pointer to the variable which holds current buffer size
  * @retval status
  */
uint8_t  usbd_audio_BuffXferCplt (uint8_t Direction, uint8_t** pbuf, uint32_t* pSize)
{
  if (Direction)
  {/* In transfer management */
    
#if (AUDIO_IN_ENABLED == 1)     
    /* Set the Isochrounous Buffer Out linked chain parameters */
    IsocInBufDesc[IsocInWrBufDescIdx].Size = IsocInCurPacketSze;
    IsocInBufDesc[IsocInWrBufDescIdx].Next = (uint8_t*)(IsocInWrPtr + IsocInCurPacketSze);
    //IsocInCurPacketSze = IsocInBufDesc[IsocInWrBufDescIdx].Size; ///to be used in futur for synchronization.
    
    /* Check if the end of the the buffer has been reached */
    if (IsocInWrBufDescIdx >= IN_PACKET_NUM - 1)
    {
      /* All buffers are full: roll back */
      IsocInWrPtr = IsocInBuff;
      IsocInBufDesc[IN_PACKET_NUM - 1].Next = IsocInBuff;
      IsocInWrBufDescIdx = 0;
    }
    else
    {
      /* Increment the write buffer pointer */
      IsocInWrPtr += IsocInCurPacketSze; //IsocInBufDesc[IsocInWrBufDescIdx].Size;
    
      /* Increment the linked chain member index */
      IsocInWrBufDescIdx++;
    }
    
    /* Check and update the buffering state machine */
    switch (IsocInWrState)
    {
      case STATE_IDLE:
        IsocInWrState = STATE_BUFFERING;
        break;
        
      case STATE_BUFFERING:
        /* Check if the buffer index reached the half of total IN buffer */
        if (IsocInWrBufDescIdx >= (IN_PACKET_NUM / 2))
        {
          /* Switch current state to Ready: allow Read operation from the buffer */
          IsocInWrState = STATE_READY;
        }
        break;
        
      case STATE_READY:   
        break;
       
      case STATE_CLOSING:        
        break;
        
      default:
        break;
    }
    
    /* Update the buffer address and number of data to be written */
    *pbuf = (uint8_t*)(IsocInWrPtr);
    *pSize = IsocInCurPacketSze;
#endif /* AUDIO_IN_ENABLED */
  }
  
  else /* if (Direction == 0) */
  {/* OUT transfer events management */
   
    if (IsocOutRdState & STATE_RUN)
    {
      ///Uncomment this line to visualize the shift between SOF and I2S clock
      STM_EVAL_LEDToggle(LED2);
      
      ///Could use STM32F2xx DMA Double Buffering feature to enhance performance ?
      if (IsocOutRdBufDescIdx == 0)
      {
        /* Start playing received packet */
        AUDIO_OUT_fops.AudioCmd(IsocOutBufDesc[OUT_PACKET_NUM - 1].Next,       /* Samples buffer pointer */
                                IsocOutBufDesc[IsocOutRdBufDescIdx].Size,      /* Number of samples in Bytes */
                                AUDIO_CMD_PLAY);                               /* Command to be processed */
        
        /* Set the current read pointer */
        IsocOutRdPtr = IsocOutBufDesc[OUT_PACKET_NUM - 1].Next;
      }
      else
      {
        /* Start playing received packet */
        AUDIO_OUT_fops.AudioCmd(IsocOutBufDesc[IsocOutRdBufDescIdx - 1].Next,  /* Samples buffer pointer */
                                IsocOutBufDesc[IsocOutRdBufDescIdx].Size,      /* Number of samples in Bytes */
                                AUDIO_CMD_PLAY);                               /* Command to be processed */
        
        /* Set the current read pointer */
        IsocOutRdPtr = IsocOutBufDesc[IsocOutRdBufDescIdx - 1].Next;
        
        /* Reset the SRC index */
        //SRCIdxUp = 0;
        //SRCIdxDown = 0;
      }    

      /* Increment the read buffer descriptor index */
      IsocOutRdBufDescIdx++;  
      
      /* Check if the end of the the buffer has been reached */
      if (IsocOutRdBufDescIdx == OUT_PACKET_NUM)
      {
        /* All buffers are full: roll back */
        IsocOutRdBufDescIdx = 0;
      }
    }
  }
  
  /* Return 0 when all opeartion is complete */
  return 0;
}

/**
  * @brief  usbd_audio_SOF
  *         Start Of Frame event management
  * @param  pdev: instance
  * @retval status
  */
static uint8_t  usbd_audio_SOF (void *pdev)
{     
  ///Uncomment this line to visualize the shift between SOF and audio clock
  STM_EVAL_LEDToggle(LED1);
  
  /* Check on current state */
  if (IsocOutWrState == STATE_READY)
  {
    /* Increment the empty frame counter */
    OutEmptyFrameCount ++;   

    /* Check if the output read operation has been started */
    if (IsocOutRdState == STATE_IDLE)
    {    
      /* Start playing received packet */
      AUDIO_OUT_fops.AudioCmd(IsocOutBuff,                               /* Samples buffer pointer */
                              IsocOutBufDesc[IsocOutRdBufDescIdx].Size,  /* Number of samples in Bytes */
                              AUDIO_CMD_PLAY);                           /* Command to be processed */   
      
      /* Set the current read pointer */
      IsocOutRdPtr = IsocOutBufDesc[IsocOutRdBufDescIdx].Next; 
      
      /* Increment the read buffer descriptor index */
      IsocOutRdBufDescIdx++;
      
      /* Update the out read operation state */
      IsocOutRdState = STATE_RUN;
      
#if (AUDIO_SYNCHRO_FEEDBACK_ENABLED == 1)
      /* Flush endpoint*/
      DCD_EP_Flush(pdev, AUDIO_FB_EP);
      
      /* Prepare data to be sent for the feedback endpoint */
      DCD_EP_Tx (pdev,
                 AUDIO_FB_EP,
                 pFBBuf,
                 FB_PCKT_SZE);
#endif /* AUDIO_SYNCHRO_FEEDBACK_ENABLED */
      
      /* Reset the SRC index */
      //SRCIdxUp = 0;
      //SRCIdxDown = 0;
    }
  }
  
  /* Check if the empty frame counter reached the threshold (means audio streaming stopped) */
  if (OutEmptyFrameCount >= EMPTY_FRAME_THRESHOLD)
  {
    IsocOutWrState = STATE_CLOSING;
    OutEmptyFrameCount = 0;
  }
  
  /* Check if there are available data in stream buffer.
    The play operation must be executed as soon as possible after the SOF detection. */
  if (/*(IsocOutWrState == STATE_CLOSING) && */
      (IsocOutRdPtr == IsocOutWrPtr) && 
      (IsocOutRdState != STATE_IDLE))
  {           
    /* Pause the audio stream */
    AUDIO_OUT_fops.AudioCmd(IsocOutBufDesc[IsocOutRdBufDescIdx - 1].Next,  /* Samples buffer pointer */
                            IsocOutBufDesc[IsocOutRdBufDescIdx].Size,      /* Number of samples in Bytes */
                            AUDIO_CMD_PAUSE);                              /* Command to be processed */
    
    IsocOutWrState = STATE_IDLE;
    IsocOutRdState = STATE_IDLE;
    
    /* Reset buffer pointers */
    IsocOutRdPtr = IsocOutBuff;
    IsocOutWrPtr = IsocOutBuff;
    IsocOutRdBufDescIdx = 0;
    IsocOutWrBufDescIdx = 0;

  #ifdef AUDIO_SYNCHRO_SRC_ENABLED
    ratio = RATIO_1;
    /* Reset the SRC synchro state machine */
    //SYNCH_SRC_Process (0, 0, 0);
    AUDIO_CORR_Out_Process(0,0,0);
  #endif /* AUDIO_SYNCHRO_SRC_ENABLED */     
  }

#if (AUDIO_IN_ENABLED == 1) 
  /* Check on current state */
  if (IsocInWrState == STATE_READY)
  {
    /* Increment the empty frame counter */
    InEmptyFrameCount ++;   
    
    /* Check if the output read operation has been started */
    if (IsocInRdState == STATE_IDLE)
    {      
      /* Update the out read operation state */
      IsocInRdState = STATE_RUN;
    }
  }

  /* Check if the empty frame counter reached the threshold (means audio streaming stopped) */
  if (InEmptyFrameCount >= EMPTY_FRAME_THRESHOLD)
  {
    /* Flush endpoint */
    //DCD_EP_Flush(pdev, AUDIO_IN_EP); ///Should be removed ??

    /* Stop recording */
    AUDIO_IN_fops.AudioCmd(IsocInBuff,                                     /* Samples buffer pointer */
                           IsocInCurPacketSze,                             /* Number of samples in Bytes */
                           AUDIO_IN_CMD_STOP);                             /* Command to be processed */
    
    IsocInWrState = STATE_IDLE;
    IsocInRdState = STATE_IDLE;
    
    /* Reset buffer pointers */
    IsocInRdPtr = IsocInBuff;
    IsocInWrPtr = IsocInBuff;
    IsocInRdBufDescIdx = 0;
    IsocInWrBufDescIdx = 0;
    InEmptyFrameCount = 0;
    
    /* Toggle the frame index */  
    ((USB_OTG_CORE_HANDLE*)pdev)->dev.in_ep[AUDIO_IN_EP & 0x7F].even_odd_frame = 
      (((USB_OTG_CORE_HANDLE*)pdev)->dev.in_ep[AUDIO_IN_EP & 0x7F].even_odd_frame)? 0:1;
      
      /* Prepare next buffer to be sent: dummy data */
      DCD_EP_Tx (pdev,
                 AUDIO_IN_EP,
                 IsocInBuffDummy,
                 IsocInCurPacketSze);
  } 
#endif /* AUDIO_IN_ENABLED */
  
  return USBD_OK;
}

/**
* @brief  usbd_audio_IN_Incplt
*         Handle the iso in incomplete event
* @param  pdev: instance
* @retval status
*/
static uint8_t  usbd_audio_IN_Incplt (void  *pdev)
{
  USB_OTG_DSTS_TypeDef dsts;  
  __IO USB_OTG_DEPCTL_TypeDef depctl;
  USB_OTG_CORE_HANDLE *ppdev = pdev; 
  dsts.d32 = USB_OTG_READ_REG32(&ppdev->regs.DREGS->DSTS);
  
#if (AUDIO_SYNCHRO_FEEDBACK_ENABLED == 1)
  depctl.d32  = USB_OTG_READ_REG32(&ppdev->regs.INEP_REGS[AUDIO_FB_EP & 0x7F]->DIEPCTL);
  
  /* Check if this is the first packet */
  if (((dsts.b.soffn) & 1) == depctl.b.dpid)
  {      
    /* EP disable */       
    depctl.b.snak = 1;
    depctl.b.epdis = 1;
    USB_OTG_WRITE_REG32(&ppdev->regs.INEP_REGS[AUDIO_FB_EP & 0x7F]->DIEPCTL, depctl.d32);
    
    /*flush endpoint*/
    DCD_EP_Flush(ppdev, AUDIO_FB_EP);
    
    /*Restart transfer  (EVEN/ODD frame scheduling will be done here when preparing packet) */      
    DCD_EP_Tx (ppdev,
               AUDIO_FB_EP,
               pFBBuf,
               FB_PCKT_SZE);
  }   
#endif /* AUDIO_SYNCHRO_FEEDBACK_ENABLED */

#if (AUDIO_IN_ENABLED == 1)     
  depctl.d32  = USB_OTG_READ_REG32(&ppdev->regs.INEP_REGS[AUDIO_IN_EP & 0x7F]->DIEPCTL);
  
  /* Check if this is the first packet */
  if (((dsts.b.soffn) & 1) == depctl.b.dpid)
  {      
    /* EP disable */       
    depctl.b.snak = 1;
    depctl.b.epdis = 1;
    USB_OTG_WRITE_REG32(&ppdev->regs.INEP_REGS[AUDIO_IN_EP & 0x7F]->DIEPCTL, depctl.d32);
    
    /* Flush endpoint */
    DCD_EP_Flush(pdev, AUDIO_IN_EP);
    
    /* Check the status of write buffer */
    if (IsocInWrState == STATE_READY)
    {
      /* Prepare next buffer to be sent: dummy data */
        /* Prepare next buffer to be sent */
        DCD_EP_Tx (pdev,
                   AUDIO_IN_EP,
                   IsocInWrPtr,
                   IsocInBufDesc[IsocInRdBufDescIdx].Size);
    }
    else
    {
      /* Prepare next buffer to be sent: dummy data */
      DCD_EP_Tx (pdev,
                 AUDIO_IN_EP,
                 IsocInBuffDummy,
                 IsocInCurPacketSze);
    }
  }   
#endif /* AUDIO_IN_ENABLED */
  
  return USBD_OK;
}
/**
* @brief  usbd_audio_OUT_Incplt
*         Handle the iso out incomplete event
* @param  pdev: instance
* @retval status
*/
static uint8_t  usbd_audio_OUT_Incplt (void  *pdev)
{
#ifdef USB_OTG_HS_CORE
//  USB_OTG_DSTS_TypeDef dsts;  
//  __IO USB_OTG_DEPCTL_TypeDef depctl;
//  USB_OTG_CORE_HANDLE *ppdev = pdev; 
//  
//  dsts.d32 = USB_OTG_READ_REG32(&ppdev->regs.DREGS->DSTS);
//  depctl.d32  = USB_OTG_READ_REG32(&ppdev->regs.OUTEP_REGS[AUDIO_OUT_EP & 0x7F]->DOEPCTL);
//  
//  /* Check if this is the first packet */
//  if (((dsts.b.soffn) & 1) == depctl.b.dpid)
//  {     
//    /* EP disable */       
//    ///depctl.b.snak = 1;
//    depctl.b.epdis = 1;
//    USB_OTG_WRITE_REG32(&ppdev->regs.OUTEP_REGS[AUDIO_OUT_EP & 0x7F]->DOEPCTL, depctl.d32);
//    
//    ///    /*flush endpoint*/
//    //    DCD_EP_Flush(pdev, AUDIO_OUT_EP);
//    //    
//    //    /* Prepare Out endpoint to receive audio data */
//    //    DCD_EP_PrepareRx(pdev,
//    //                     AUDIO_OUT_EP,
//    //                     IsocOutBuff,
//    //                     IsocOutCurPacketSze);
//  }     
#endif /* USB_OTG_FS_CORE */
  return USBD_OK;
}

/******************************************************************************
     AUDIO correction mechanism management
******************************************************************************/
/**
  * @brief  AUDIO_CORR_Out_GetCurrent
  *         Get the current shift and direction values between input and output streams.
  * @param  pValue: value of the shift to be corrected.
  * @param  pDirection: direction of the correction (0:overrun or 1:underrun)
  * @param  pBuff: Option pointer
  * @retval 0: there are available data to process. !0: no available data to process.
  */
uint32_t AUDIO_CORR_Out_GetCurrent(uint32_t* pValue, uint32_t* pDirection, uint32_t** pBuff)
{
  uint8_t* pRdPtr = IsocOutRdPtr;
  //  static uint32_t currDistanceOut = 0;  ///this varaible is declared global only for debug purposes.
  
  /* Check if the player is already running */
  if (IsocOutRdState & STATE_RUN)
  {
    /* Update the read pointer image */ /// could use DMA get count for more precision ? Doesn't seem necessary.
    pRdPtr = IsocOutRdPtr; //+ (IsocOutBufDesc[IsocOutRdBufDescIdx-1].Size - (Audio_MAL_GetRemCount() * (IsocOutCurBitRes/8)));
    
    /* Provide the pointer to the parameter to be updated */
    switch (AUDIO_OUT_fops.pOptions[0])
    {
#if (AUDIO_SYNCHRO_FEEDBACK_ENABLED == 1)
    case SYNCH_FEEDBACK:
      *pBuff = IsocOutCurSampleNbrTb;
      break;
#endif /* AUDIO_SYNCHRO_FEEDBACK_ENABLED */
    
#if (AUDIO_SYNCHRO_SRC_ENABLED == 1)    
    case SYNCH_SRC:
      *pBuff = (uint32_t*)&ratio;
      break;
#endif /* AUDIO_SYNCHRO_SRC_ENABLED */
      
    default:
      *pBuff = &IsocOutCurPacketSze;
      break; 
    }
    
    /* Check if roll-back has been detected */
    if (((uint32_t)IsocOutWrPtr) >= (uint32_t)pRdPtr)
    {/* No roll-back */
      currDistanceOut = (uint32_t)(((uint32_t)IsocOutWrPtr) - (uint32_t)pRdPtr);
    }
    else
    {/* Roll-back occured */
      currDistanceOut = (uint32_t)(((uint32_t)(IsocOutBufDesc[OUT_PACKET_NUM-2].Next) +  \
                                  IsocOutBufDesc[OUT_PACKET_NUM-1].Size) - \
                                  (uint32_t)pRdPtr + ((uint32_t)IsocOutWrPtr) - \
                                  (uint32_t)(IsocOutBufDesc[OUT_PACKET_NUM-1].Next));    
    }

    /* Distance between pointers is increasing */
    if (currDistanceOut < (((uint32_t)(IsocOutBufDesc[OUT_PACKET_NUM-2].Next) +  \
                        IsocOutBufDesc[OUT_PACKET_NUM-1].Size -\
                        (uint32_t)IsocOutBuff) / 2))
    {
      /* Update the reading State */
      IsocOutRdState = STATE_RUN_OVR;
      *pDirection = 0;
      
      /* Update the shift value */
      *pValue = ((((uint32_t)(IsocOutBufDesc[OUT_PACKET_NUM-2].Next) +  \
                        IsocOutBufDesc[OUT_PACKET_NUM-1].Size -\
                        (uint32_t)IsocOutBuff) / 2) - currDistanceOut) / 4; // / IsocOutCurPacketSze;      
    }
    /* Distance between pointers is decreasing */
    else if (currDistanceOut > (((uint32_t)(IsocOutBufDesc[OUT_PACKET_NUM-2].Next) +  \
                        IsocOutBufDesc[OUT_PACKET_NUM-1].Size -\
                        (uint32_t)IsocOutBuff) / 2))
    {
      /* Update the reading State */
      IsocOutRdState = STATE_RUN_UDR; 
      *pDirection = 1;
      
      /* Update the shift value */
      *pValue = (currDistanceOut - (((uint32_t)(IsocOutBufDesc[OUT_PACKET_NUM-2].Next) +  \
                        IsocOutBufDesc[OUT_PACKET_NUM-1].Size -\
                        (uint32_t)IsocOutBuff) / 2)) /4; /// IsocOutCurPacketSze;    
    }
    /* Distance between pointers is stable (no shift) */      
    else
    {
      /* Update the shift value */        
      *pValue = 0; //currDistanceOut;
      
      /* No data to process */
      return 0;
    }

#if (AUDIO_SYNCHRO_FEEDBACK_ENABLED == 1)
    /* Update Feedback information */
    pFBBuf[0] = FEEDBACK_FF_FS_B2(IsocOutCurSampleNbrTb[0], IsocOutCurSampleNbrTb[1]);
    pFBBuf[1] = FEEDBACK_FF_FS_B1(IsocOutCurSampleNbrTb[0], IsocOutCurSampleNbrTb[1]);
    pFBBuf[2] = FEEDBACK_FF_FS_B0(IsocOutCurSampleNbrTb[0], IsocOutCurSampleNbrTb[1]);
#endif /* AUDIO_SYNCHRO_FEEDBACK_ENABLED */  
    
    /* Return 1 to inform that there are avaialble data to process */
    return 1;         
  }
  else /* The player is stopped */
  {
    /* No data to process */
    return 0;
  }
}

/**
  * @brief  AUDIO_CORR_In_GetCurrent
  *         Get the current shift and direction values between input and output streams.
  * @param  pValue: value of the shift to be corrected.
  * @param  pDirection: direction of the correction (0:overrun or 1:underrun)
  * @param  pBuff: Option pointer
  * @retval 0: there are available data to process. !0: no available data to process.
  */
uint32_t AUDIO_CORR_In_GetCurrent(uint32_t* pValue, uint32_t* pDirection, uint32_t** pBuff)
{
  uint8_t* pRdPtr = IsocOutRdPtr;
  //  static uint32_t currDistanceIn = 0;  ///this varaible is declared global only for debug purposes.
  
  /* Check if the player is already running */
  if (IsocInWrState == STATE_READY)
  {
    /* Update the read pointer image */
    pRdPtr = IsocInRdPtr;
    
    /* Provide the pointer to the parameter to be updated */
    *pBuff = &IsocInCurPacketSze;   /* No impact in this case */
    
    /* Check if roll-back has been detected */
    if (((uint32_t)IsocInWrPtr) >= (uint32_t)pRdPtr)
    {/* No roll-back */
      currDistanceIn = (uint32_t)(((uint32_t)IsocInWrPtr) - (uint32_t)pRdPtr);
    }
    else
    {/* Roll-back occured */
      currDistanceIn = (uint32_t)(((uint32_t)(IsocInBufDesc[IN_PACKET_NUM-2].Next) +  \
                                  IsocInBufDesc[IN_PACKET_NUM-1].Size) - \
                                  (uint32_t)pRdPtr + ((uint32_t)IsocInWrPtr) - \
                                  (uint32_t)(IsocInBufDesc[IN_PACKET_NUM-1].Next));    
    }

    /* Distance between pointers is increasing */
    if (currDistanceIn < (((uint32_t)(IsocInBufDesc[IN_PACKET_NUM-2].Next) +  \
                        IsocInBufDesc[IN_PACKET_NUM-1].Size -\
                        (uint32_t)IsocInBuff) / 2))
    {
      /* Update the reading State */
      //IsocInRdState = STATE_RUN_OVR;
      *pDirection = 0;
      
      /* Update the shift value */
      *pValue = ((((uint32_t)(IsocInBufDesc[IN_PACKET_NUM-2].Next) +  \
                        IsocInBufDesc[IN_PACKET_NUM-1].Size -\
                        (uint32_t)IsocInBuff) / 2) - currDistanceIn) / IsocInCurPacketSze;      
    }
    /* Distance between pointers is decreasing */
    else if (currDistanceIn > (((uint32_t)(IsocInBufDesc[IN_PACKET_NUM-2].Next) +  \
                        IsocInBufDesc[IN_PACKET_NUM-1].Size -\
                        (uint32_t)IsocInBuff) / 2))
    {
      /* Update the reading State */
      //IsocOutRdState = STATE_RUN_UDR; 
      *pDirection = 1;
      
      /* Update the shift value */
      *pValue = (currDistanceIn - (((uint32_t)(IsocInBufDesc[IN_PACKET_NUM-2].Next) +  \
                        IsocInBufDesc[IN_PACKET_NUM-1].Size -\
                        (uint32_t)IsocInBuff) / 2)) / IsocInCurPacketSze;    
    }
    /* Distance between pointers is stable (no shift) */      
    else
    {
      /* Update the shift value */        
      *pValue = currDistanceIn;
      
      /* No data to process */
      return 0;
    }
    
    /* Return 1 to inform that there are avaialble data to process */
    return 1;         
  }
  else /* The player is stopped */
  {
    /* No data to process */
    return 0;
  }
}

/******************************************************************************
     AUDIO Class requests management
******************************************************************************/

#if defined(USB_AUDIO_V1_0)
/**
  * @brief  AUDIO_Req_GetCurrent
  *         
  * @param  pdev: instance
  * @param  req: setup class request
  * @retval status
  */
static void AUDIO_Req_GetCurrent(void *pdev, USB_SETUP_REQ *req)
{  
  /* Send the current mute state */
  USBD_CtlSendData (pdev, 
                    AudioCtl,
                    req->wLength);
}

/**
  * @brief  AUDIO_Req_SetCurrent
  *         
  * @param  pdev: instance
  * @param  req: setup class request
  * @retval status
  */
static void AUDIO_Req_SetCurrent(void *pdev, USB_SETUP_REQ *req)
{ 
  if (req->wLength)
  {
    /* Prepare the reception of the buffer over EP0 */
    USBD_CtlPrepareRx (pdev, 
                       AudioCtl,
                       req->wLength);
    
    /* Set the global variables indicating current request and its length 
    to the function usbd_audio_EP0_RxReady() which will process the request */
    AudioCtlCmd = AUDIO_REQ_SET_CUR;     /* Set the request value */
    AudioCtlLen = req->wLength;          /* Set the request data length */
    AudioCtlUnit = HIBYTE(req->wIndex);  /* Set the request target unit */
    AudioTrgtEpt = LOBYTE(req->wIndex);  /* Set the request target endpoint */
  }
}
#elif defined(USB_AUDIO_V2_0)
/**
  * @brief  AUDIO_Req_Get
  *         
  * @param  pdev: instance
  * @param  req: setup class request
  * @retval status
  */
static void AUDIO_Req_Get(void *pdev, USB_SETUP_REQ *req)
{  
  uint8_t len = 0;
  
  /* Target is Interface --------------------------------*/
  if (req->bmRequest & 0x01)
  {
    /* Check which enity ID is target */
    switch (HIBYTE(req->wIndex))
    {
      /* Input Terminal: No control supported */
      case IN_TERMINAL_ID:
        USBD_CtlError (pdev, req);
        return;

      /* Feature Unit: Mute control supported */
      case OUT_FU_ID:
        if (req->bRequest == AUDIO_REQ_CUR)
        {
          /* Return the current state of Mute control */
          AudioCtl[0] = MuteCtrl;
          len = 1;
        }
        else if (req->bRequest == AUDIO_REQ_RANGE)
        {
          /* Return the range of Mute control */
          AudioCtl[0] = 0x01; /* wNumSubRanges (0) */
          AudioCtl[1] = 0x00; /* wNumSubRanges (1) */
          AudioCtl[2] = 0x00; /* bMIN */
          AudioCtl[3] = 0x01; /* bMAX */
          AudioCtl[4] = 0x00; /* bRES */
          len = 5;
        }
        break;

      /* Output terminal: No control supported */
      case OUT_TERMINAL_ID:
        USBD_CtlError (pdev, req);
        return;
      
      /* Clock Source: Sampling Frequency control supported */
      case OUT_CLOCK_ID:
        if (req->bRequest == AUDIO_REQ_CUR)
        {
          /* Return the current Sampling Frequency control */
          AudioCtl[0] = BYTE_0(IsocOutCurSampleFreq);
          AudioCtl[1] = BYTE_1(IsocOutCurSampleFreq);
          AudioCtl[2] = BYTE_2(IsocOutCurSampleFreq);
          AudioCtl[3] = BYTE_3(IsocOutCurSampleFreq);
          len = 4;
        }
        else if (req->bRequest == AUDIO_REQ_RANGE)
        {
          /* Return the range of Sampling Frequency control */
      #if (SUPPORTED_FREQ_NBR == 1)
          AudioCtl[0]  = 0x01; /* wNumSubRanges (0) */
          AudioCtl[1]  = 0x00; /* wNumSubRanges (1) */       
          AudioCtl[2]  = BYTE_0(DEFAULT_OUT_AUDIO_FREQ); /* bMIN (0) */
          AudioCtl[3]  = BYTE_1(DEFAULT_OUT_AUDIO_FREQ); /* bMIN (1) */
          AudioCtl[4]  = BYTE_2(DEFAULT_OUT_AUDIO_FREQ); /* bMIN (2) */
          AudioCtl[5]  = BYTE_3(DEFAULT_OUT_AUDIO_FREQ); /* bMIN (3) */
          AudioCtl[6]  = BYTE_0(DEFAULT_OUT_AUDIO_FREQ); /* bMAX (0) */
          AudioCtl[7]  = BYTE_0(DEFAULT_OUT_AUDIO_FREQ); /* bMAX (1) */
          AudioCtl[8]  = BYTE_0(DEFAULT_OUT_AUDIO_FREQ); /* bMAX (2) */
          AudioCtl[9]  = BYTE_0(DEFAULT_OUT_AUDIO_FREQ); /* bMAX (3) */
          AudioCtl[10] = 0x00; /* bRES (0) */
          AudioCtl[11] = 0x00; /* bRES (1) */
          AudioCtl[12] = 0x00; /* bRES (2) */
          AudioCtl[13] = 0x00; /* bRES (3) */
     #else
       #error "ERROR: Multiple sampling frequecies not yet supported!"
     #endif /* SUPPORTED_FREQ_NBR */
          len = 14;
        }
        break;
      
      /* Unsupported target */
      default:
        USBD_CtlError (pdev, req);
        return;
    }
    
  }
  
  /* Target is endpoint ---------------------------------*/
  else
  {
    /* No endpoint requests are supported */
    USBD_CtlError (pdev, req);
    return;    
  }
  
  /* Send the required data -----------------------------*/
  USBD_CtlSendData (pdev, 
                    AudioCtl,
                    len);
}

/**
  * @brief  AUDIO_Req_Set
  *         
  * @param  pdev: instance
  * @param  req: setup class request
  * @retval status
  */
static void AUDIO_Req_Set(void *pdev, USB_SETUP_REQ *req)
{ 
  if (req->wLength)
  {
    /* Prepare the reception of the buffer over EP0 */
    USBD_CtlPrepareRx (pdev, 
                       AudioCtl,
                       req->wLength);
    
    /* Set the global variables indicating current request and its length 
    to the function usbd_audio_EP0_RxReady() which will process the request */
    AudioCtlCmd = req->bRequest;         /* Set the request value */
    AudioCtlLen = req->wLength;          /* Set the request data length */
    AudioCtlUnit = HIBYTE(req->wIndex);  /* Set the request target unit */
    AudioTrgtEpt = LOBYTE(req->wIndex);  /* Set the request target endpoint */
  }
}
#else
 #error "ERROR: Missing define of USB_AUDIO_V1_0 or USB_AUDIO_V2_0 in usbd_audio_core.h!"
#endif /* USB_AUDIO_V1_0 */





/**
  * @brief  usbd_audio_DescUpdate
  *         
  * @param  FBstate: Feedback pipe state
  * @retval None
  */
void usbd_audio_DescUpdate(uint32_t FBstate)
{
  uint32_t idx = 0;
#if (AUDIO_SYNCHRO_FEEDBACK_ENABLED == 1)  
  /* If Feedback endpoint is active */
  if (FBstate == SYNCH_FEEDBACK)
  {
 #if defined (USB_AUDIO_V1_0)       
    /* Update the number of endpoints and their references */
    while (idx++ < AUDIO_CONFIG_DESC_SIZE)
    {      
      /* Update the reference of the feedback endpoint*/ 
      if ((usbd_audio_CfgDesc[idx] == AUDIO_STANDARD_ENDPOINT_DESC_SIZE) && \
               (usbd_audio_CfgDesc[idx + 1] == USB_ENDPOINT_DESCRIPTOR_TYPE) && \
               (usbd_audio_CfgDesc[idx + 2] == AUDIO_OUT_EP) && \
               (usbd_audio_CfgDesc[idx + 3] == 0x05))
      {
        /* Reference to the feedback endpoint */
        usbd_audio_CfgDesc[idx + 8] = AUDIO_FB_EP;
      }
    }
 #endif /* USB_AUDIO_V1_0 */
  }
  
  /* If Feedback endpoint is not active */
  else
#endif /* AUDIO_SYNCHRO_FEEDBACK_ENABLED */
  {
#if defined (USB_AUDIO_V1_0)
    /* Update the number of endpoints and their references */
    while (idx++ < AUDIO_CONFIG_DESC_SIZE)
    {      
      /* Update the reference of the feedback endpoint*/ 
      if ((usbd_audio_CfgDesc[idx] == AUDIO_STANDARD_ENDPOINT_DESC_SIZE) && \
               (usbd_audio_CfgDesc[idx + 1] == USB_ENDPOINT_DESCRIPTOR_TYPE) && \
               (usbd_audio_CfgDesc[idx + 2] == AUDIO_OUT_EP) && \
               (usbd_audio_CfgDesc[idx + 3] == 0x05))
      {
        /* Reference to the feedback endpoint */
        usbd_audio_CfgDesc[idx + 8] = 0;
      }
    } 
#endif /* USB_AUDIO_V1_0 */
  }

  /* Update the product release depending on the the synchronization method */
  USBD_DeviceDesc[12] = FBstate;
  
#ifdef USE_USB_OTG_HS 
  /// TO BE UPDATED-------
  /// ...
  ///---------------------
#endif /* USE_USB_OTG_HS */
}

/**
  * @brief  USBD_audio_GetCfgDesc 
  *         Returns configuration descriptor.
  * @param  speed : current device speed
  * @param  length : pointer data length
  * @retval pointer to descriptor buffer
  */
static uint8_t  *USBD_audio_GetCfgDesc (uint8_t speed, uint16_t *length)
{
  *length = sizeof (usbd_audio_CfgDesc);
  return usbd_audio_CfgDesc;
}

/**
  * @}
  */ 

/**
  * @}
  */ 

/**
  * @}
  */ 

/******************* (C) COPYRIGHT 2012 STMicroelectronics *****END OF FILE****/
