/**
  ******************************************************************************
  * @file    stm32f4_discovery_audio_recorder.c
  * @author  MCD Application Team
  * @version V0.1.0
  * @date    05-January-2012
  * @brief   This file contains the main function for audio recording using MEMS 
  *          microphone embedded in the STM32F4-Discovery board.
  *         
  *         This driver is not functional: the data received from the MEMS Microphone
  *         need to be processed with decimation filter.
  *         To impelement the decimation filter please refer and use the following
  *         functions:
  *            AUDIO_REC_SPI_IRQHANDLER()
  *            STM32_AudioRec_GetSample()
  *            (optionally) STM32_AudioRec_DataProcessBuffer()
  *          
  ******************************************************************************          
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2011 STMicroelectronics</center></h2>
  ******************************************************************************  
  */

/* Includes ------------------------------------------------------------------*/
#include "stm32f4_discovery_audio_recorder.h"

/** @addtogroup Audio_Utilities
  * @{
  */

/** @defgroup AUDIO_RECORDER
  * @brief    This file contains the main function for audio recording using ADC
  *           and timer on STM322xG-EVAL evaluation board.
  * @{
  */

/** @defgroup AUDIO_RECORDER_Private_Types
  * @{
  */
typedef enum
{
  LittleEndian,
  BigEndian
}Endianness;
/**
  * @}
  */

/** @defgroup AUDIO_RECORDER_Private_Defines
  * @{
  */
/**
  * @}
  */

/** @defgroup AUDIO_RECORDER_Private_Macros
  * @{
  */
/**
  * @}
  */

/** @defgroup AUDIO_RECORDER_Private_Variables
  * @{
  */

/* Audio recording Samples format (from 8 to 16 bits) */
uint32_t AudioRecBitRes = 16; 

/* Audio recording number of channels (1 for Mono or 2 for Stereo) */
uint32_t AudioRecChnlNbr = 1;

/* Main buffer pointer for the recorded data storing */
uint8_t* pAudioRecBuf;

/* Current size of the recorded buffer */
uint32_t AudioRecCurrSize = 0; 

/* Number of data currently stored into the audio buffer */
uint32_t AudioRecCounter = 0;

/* Current Timer autoreload value */
uint32_t AutoReload     = 0xFFFFFFFF;
uint32_t OrigAutoReload = 0xFFFFFFFF;
int32_t  extAutoReload  = 0;

/* Current state of the audio recorder interface intialization */
static uint32_t AudioRecInited = 0;

/* Local buffer for temporary operations */
uint8_t* TmpBuff = 0;

/* Temporary data sample */
static uint16_t SampleN = 0;
static uint16_t InternalBuffer[INTERNAL_BUFF_SIZE];
static uint32_t InternalBufferSize = 0;

/**
  * @}
  */

/** @defgroup AUDIO_RECORDER_Private_FunctionPrototypes
  * @{
  */
static void STM32_AudioRec_GPIO_Init(void);
static void STM32_AudioRec_SPI_Init(uint32_t Freq);
static void STM32_AudioRec_TIM_Init(uint32_t AudioFreq);
static void STM32_AudioRec_NVIC_Init(void);

/**
  * @}
  */

/** @defgroup AUDIO_RECORDER_Private_Functions
  * @{
  */

/**
  * @brief  Initialize wave recording
  * @param  None
  * @retval None
  */
uint32_t STM32_AudioRec_Init(uint32_t AudioFreq, uint32_t BitRes, uint32_t ChnlNbr)
{
  /* Check if the interface is already initialized */
  if (AudioRecInited)
  {
    /* No need for initialization */
    return 0;
  }
  else
  {
    /* Configure the GPIOs */
    STM32_AudioRec_GPIO_Init();
    
    /* Configure the interrupts (for timer) */
    STM32_AudioRec_NVIC_Init();
    
    /* Configure the SPI */
    STM32_AudioRec_SPI_Init(AudioFreq);
    
    /* Configure the Timer which triggers the data conversion */
    STM32_AudioRec_TIM_Init(AudioFreq);
    
    /* Set the local parameters */
    AudioRecBitRes = BitRes;
    AudioRecChnlNbr = ChnlNbr;
    
    /* Set state of the audio recorder to initialized */
    AudioRecInited = 1;
    
    /* Return 0 if all operations are OK */
    return 0;
  }
}

/**
  * @brief  Start audio recording
  * @param  None
  * @retval None
  */
uint32_t STM32_AudioRec_Start(uint8_t* pbuf, uint32_t size)
{
  /* Check if the interface has already been initialized */
  if (AudioRecInited)
  {
    /* Store the location and size of the audio buffer */
    pAudioRecBuf = pbuf;
    AudioRecCurrSize = size;
    
    /* The Data transfer is performed in the SPI interrupt routine */
    /* Enable the SPI peripheral */
    I2S_Cmd(SPI2, ENABLE); 
    
    /* Enable AUDIO_REC_TIM interrupts */
    TIM_ITConfig(AUDIO_REC_TIM, TIM_IT_Update, ENABLE);
    
    /* Start AUDIO_REC_TIM */
    TIM_Cmd(AUDIO_REC_TIM, ENABLE);
    
    /* Return 0 if all operations are OK */
    return 0;
  }
  else
  {
    /* Cannot perform operation */
    return 1;
  }
}

/**
  * @brief  Stop audio recording
  * @param  None
  * @retval None
  */
uint32_t STM32_AudioRec_Stop(void)
{
  /* Check if the interface has already been initialized */
  if (AudioRecInited)
  {
    /* Disable AUDIO_REC_TIM interrupts */
    TIM_ITConfig(AUDIO_REC_TIM, TIM_IT_Update, DISABLE);
    
    /* Disable AUDIO_REC_TIM */
    TIM_Cmd(AUDIO_REC_TIM, DISABLE);
    
    /* Stop conversion */
    I2S_Cmd(SPI2, DISABLE); 
    
    /* Return 0 if all operations are OK */
    return 0;
  }
  else
  {
    /* Cannot perform operation */
    return 1;
  }
}

/**
  * @brief  Update the current value of the timer auto reload
  * @param  Step: decrement/increment step
  * @param  Direction: Direction of increment/decrement. Can be 
  *         AUDIO_REC_DIRECTION_DOWN or AUDIO_REC_DIRECTION_UP.
  * @retval None
  */
uint32_t STM32_AudioRec_ClkUpdate(uint32_t Step, uint32_t Direction)
{    
  /* Check if the interface has already been initialized */
  if (AudioRecInited == 0)
  {
    /* Operation cannot be performed */
    return 1;
  }
  
  /* Check on the direction of the clock update */
  if (Direction == AUDIO_REC_DIRECTION_UP)
  {
    /* Speed up the sampling rate */
    AutoReload  -= Step;
  }
  else
  {
    /* Slow down the sampling rate */
    AutoReload  += Step;
    
  }
  
  /* If operation is OK, return 0 */
  return 0;
}

/**
  * @brief  This function handles AUDIO_REC_SPI global interrupt request.
  * @param  None
  * @retval None
*/
void AUDIO_REC_SPI_IRQHANDLER(void)
{  
  /* Check if data are available in SPI Data register */
  if (SPI_GetITStatus(SPI2, SPI_I2S_IT_RXNE) != RESET)
  {
    /* Store the obtained value */
    InternalBuffer[InternalBufferSize++] = SPI_I2S_ReceiveData(SPI2);
    
    /* Check to prevent overflow condition */
    if (InternalBufferSize >= INTERNAL_BUFF_SIZE)
    {
      InternalBufferSize = 0;
      /* 
      
        YOU MAY CALL A FUNCTION OR SET A FLAG FOR PROCESSING 
        THE OBTAINED BUFFER HERE 
      
      */
    }
  }
}

/**
  * @brief  This function handles AUDIO_REC_TIM global interrupt request.
  * @param  None
  * @retval None
  */
void AUDIO_REC_TIM_IRQHANDLER(void)
{
  uint32_t idx = 0;
  static uint32_t modcount = 0;
  
  /* Clear AUDIO_REC_TIM interrupt */
  TIM_ClearITPendingBit(AUDIO_REC_TIM, TIM_IT_Update);

  /* Update auto reload value */
  TIM_SetAutoreload(AUDIO_REC_TIM, AutoReload);
  
  /* Restore original autorelaod value (reduce dynamic of clock update) */
  if (AutoReload != OrigAutoReload)
  {   
    /* Keep modified value only for 10ms then return to original vlaue */
    if (modcount++ >= 160)
    {
      AutoReload = OrigAutoReload;
      modcount = 0;
    }
  }
  
  /* Get one data sample */
  STM32_AudioRec_GetSample((uint8_t*)&SampleN, 1);
  
#ifdef AUDIO_REC_USE_DATA_PROC_SINGLE
  /* Process the aquired data (Digital filter, Encoding ...) using single data */  
  STM32_AudioRec_DataProcessSingle((uint8_t*)(&SampleN), AudioRecBitRes, AudioRecChnlNbr);
#endif /* AUDIO_REC_USE_DATA_PROC_SINGLE */
  
  /* Store the converted sample N value (same value repeated if Stereo mode enabled) */
  while (idx++ <  AudioRecChnlNbr)
  {
    /* Check the data format type */
    if (AudioRecBitRes <= 8)
    {
      /* 1-byte data format */
      ((uint8_t*)(pAudioRecBuf))[AudioRecCounter] = (uint8_t)((SampleN >> 4) & 0x00FF);
    }
    else
    {      
      /* 2-bytes data format */
      ((uint16_t*)(pAudioRecBuf))[AudioRecCounter] = (uint16_t)((SampleN));
    }
    
    /* Increment the audio counter */
    AudioRecCounter++;
  }

  /* If the current buffer is full, prepare the next buffer */
  if (AudioRecCounter >= (AudioRecCurrSize / (AudioRecBitRes/8)))
  {    
    /* Reset the audio counter */
    AudioRecCounter = 0;
    
#ifdef AUDIO_REC_USE_DATA_PROC_BUFFER     
    /* Save the current buffer location */
    TmpBuff = (uint8_t*)pAudioRecBuf;
#endif /* AUDIO_REC_USE_DATA_PROC_BUFFER  */
    
    /* Call the record update function */
    STM32_AudioRec_Update(&pAudioRecBuf, &AudioRecCurrSize);
    
#ifdef AUDIO_REC_USE_DATA_PROC_BUFFER
    /* Process the aquired data (Digital filter, Encoding ...) using entire buffer */  
    STM32_AudioRec_DataProcessBuffer(TmpBuff, AudioRecCurrSize, AudioRecBitRes, AudioRecChnlNbr);
#endif /* AUDIO_REC_USE_DATA_PROC_BUFFER */
  }
}


/**
  * @brief  STM32_AudioRec_GetSample.
  * @param  None
  * @retval None
  */
void STM32_AudioRec_GetSample(uint8_t* pBuf, uint32_t Nbr)
{
  uint32_t idx = 0;
  uint16_t data = InternalBuffer[0];
  
  //THIS IS JUST A DUMMY EXAMPLE: Data processing should be done instead of the dummy processing
  /* Get the mean value */
  for(idx = 0; idx < InternalBufferSize; idx++)
  {
    data = (data + InternalBuffer[idx]) / 2;
  }
  
  /* Reset the counter to 0 */
  InternalBufferSize = 0;
    
  /* Return the computed value */
  *(uint16_t*)pBuf = data;
}

/**
  * @brief  STM32_AudioRec_GPIO_Init.
  * @param  None
  * @retval None
  */
static void STM32_AudioRec_GPIO_Init(void)
{  
  GPIO_InitTypeDef GPIO_InitStructure;

  /* Enable GPIO clocks */
  RCC_AHB1PeriphClockCmd(SPI_SCK_GPIO_CLK | SPI_MOSI_GPIO_CLK, ENABLE);

  /* Enable GPIO clocks */
  RCC_AHB1PeriphClockCmd(SPI_SCK_GPIO_CLK | SPI_MOSI_GPIO_CLK, ENABLE);

  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

  /* SPI SCK pin configuration */
  GPIO_InitStructure.GPIO_Pin = SPI_SCK_PIN;
  GPIO_Init(SPI_SCK_GPIO_PORT, &GPIO_InitStructure);
  
  /* Connect SPI pins to AF5 */  
  GPIO_PinAFConfig(SPI_SCK_GPIO_PORT, SPI_SCK_SOURCE, SPI_SCK_AF);
  
  /* SPI MOSI pin configuration */
  GPIO_InitStructure.GPIO_Pin =  SPI_MOSI_PIN;
  GPIO_Init(SPI_MOSI_GPIO_PORT, &GPIO_InitStructure);
  GPIO_PinAFConfig(SPI_MOSI_GPIO_PORT, SPI_MOSI_SOURCE, SPI_MOSI_AF);
}

/**
  * @brief  STM32_AudioRec_SPI_Init.
  * @param  None
  * @retval None
  */
static void STM32_AudioRec_SPI_Init(uint32_t Freq)
{
  I2S_InitTypeDef I2S_InitStructure;

  /* Enable the SPI clock */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_SPI2,ENABLE);
  
  /* SPI configuration -------------------------------------------------------*/
  SPI_I2S_DeInit(SPI2);
  I2S_InitStructure.I2S_AudioFreq = 64000;
  I2S_InitStructure.I2S_Standard = I2S_Standard_MSB;
  I2S_InitStructure.I2S_DataFormat = I2S_DataFormat_16b;
  I2S_InitStructure.I2S_CPOL = I2S_CPOL_Low;
  I2S_InitStructure.I2S_Mode = I2S_Mode_MasterRx;
  I2S_InitStructure.I2S_MCLKOutput = I2S_MCLKOutput_Disable;
  /* Initialize the I2S peripheral with the structure above */
  I2S_Init(SPI2, &I2S_InitStructure);

  /* Enable the Rx buffer not empty interrupt */
  SPI_I2S_ITConfig(SPI2, SPI_I2S_IT_RXNE, ENABLE);
}

/**
  * @brief  STM32_AudioRec_TIM_Init.
  * @param  None
  * @retval None
  */
static void STM32_AudioRec_TIM_Init(uint32_t AudioFreq)
{
  /* AUDIO_REC_TIM clock enable */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3 , ENABLE);

  /* AUDIO_REC_TIM Peripheral Configuration */
  TIM_DeInit(AUDIO_REC_TIM);

  /* Set AUDIO_REC_TIM period to the audio frequency using system clock */
  AutoReload = (uint32_t)(84000000/AudioFreq);
  TIM_SetAutoreload(AUDIO_REC_TIM, AutoReload);

  /* AUDIO_REC_TIM TRGO selection */
  TIM_SelectOutputTrigger(AUDIO_REC_TIM, TIM_TRGOSource_Update);

  /* Enable AUDIO_REC_TIM interrupts */
  TIM_ITConfig(AUDIO_REC_TIM, TIM_IT_Update, ENABLE);
  
  /* Update the extended reload value */
  extAutoReload = 0;
  
  /* Save the original autoreload value */
  OrigAutoReload = AutoReload;
}

/**
  * @brief  STM32_AudioRec_NVIC_Init.
  * @param  None
  * @retval None
  */
static void STM32_AudioRec_NVIC_Init(void)
{
  NVIC_InitTypeDef NVIC_InitStructure;
  
  /* Enable the Timer Interrupt to trigger the ADC */
  NVIC_InitStructure.NVIC_IRQChannel = AUDIO_REC_TIM_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = AUDIO_REC_TIM_PRE_PRIO;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = AUDIO_REC_TIM_SUB_PRIO;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
  
  /* Configure the SPI interrupt priority */
  NVIC_InitStructure.NVIC_IRQChannel = SPI2_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = AUDIO_REC_SPI_PRE_PRIO;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = AUDIO_REC_SPI_SUB_PRIO;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
}

/**
  * @}
  */

/**
  * @}
  */
/**
  * @}
  */

/******************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE****/
