/**
  ******************************************************************************
  * @file    stm3210c_audio_recorder.c
  * @author  MCD Application Team
  * @version V5.1.0
  * @date    27-January-2012
  * @brief   This file contains the main function for audio recording using ADC
  *          and TIM timer on STM3210C-EVAL evaluation board.
  *
  *     *******************************************************************  
  *                           IMPORTANT NOTES
  *     *******************************************************************  
  *      This driver is intended for use ONLY in the scope of the current
  *      "STM32 USB Device Audio Streaming" Example.  
  *    
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * FOR MORE INFORMATION PLEASE READ CAREFULLY THE LICENSE AGREEMENT FILE
  * LOCATED IN THE ROOT DIRECTORY OF THIS FIRMWARE PACKAGE.
  *
  * <h2><center>&copy; COPYRIGHT 2012 STMicroelectronics</center></h2>
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "stm3210c_audio_recorder.h"

/** @addtogroup AUDIO_RECORDER
  * @{
  */

/** @defgroup AUDIO_RECORDER
  * @brief    This file contains the main function for audio recording using ADC
  *           and timer on STM322xG-EVAL evaluation board.
  * @{
  */

/** @defgroup AUDIO_RECORDER_Private_Types
  * @{
  */
typedef enum
{
  LittleEndian,
  BigEndian
}Endianness;
/**
  * @}
  */

/** @defgroup AUDIO_RECORDER_Private_Defines
  * @{
  */
/**
  * @}
  */

/** @defgroup AUDIO_RECORDER_Private_Macros
  * @{
  */
/**
  * @}
  */

/** @defgroup AUDIO_RECORDER_Private_Variables
  * @{
  */

/* Audio recording Samples format (from 8 to 16 bits) */
uint32_t AudioRecBitRes = 16; 

/* Audio recording number of channels (1 for Mono or 2 for Stereo) */
uint32_t AudioRecChnlNbr = 1;

/* Main buffer pointer for the recorded data storing */
uint8_t* pAudioRecBuf;

/* Current size of the recorded buffer */
uint32_t AudioRecCurrSize = 0; 

/* Number of data currently stored into the audio buffer */
uint32_t AudioRecCounter = 0;

/* Current Timer autoreload value */
uint32_t AutoReload     = 0xFFFFFFFF;
uint32_t OrigAutoReload = 0xFFFFFFFF;
int32_t  extAutoReload  = 0;

/* Current state of the audio recorder interface intialization */
static uint32_t AudioRecInited = 0;

/* Local buffer for temporary operations */
uint8_t* TmpBuff = 0;

/**
  * @}
  */

/** @defgroup AUDIO_RECORDER_Private_FunctionPrototypes
  * @{
  */
static void STM32_AudioRec_GPIO_Init(void);
static void STM32_AudioRec_ADC_Init(void);
static void STM32_AudioRec_TIM_Init(uint32_t AudioFreq);
static void STM32_AudioRec_NVIC_Init(void);


/**
  * @}
  */

/** @defgroup AUDIO_RECORDER_Private_Functions
  * @{
  */

/**
  * @brief  Initialize wave recording
  * @param  None
  * @retval None
  */
uint32_t STM32_AudioRec_Init(uint32_t AudioFreq, uint32_t BitRes, uint32_t ChnlNbr)
{
  /* Check if the interface is already initialized */
  if (AudioRecInited)
  {
    /* No need for initialization */
    return 0;
  }
  else
  {
    /* Configure the GPIOs */
    STM32_AudioRec_GPIO_Init();
    
    /* Configure the interrupts (for timer) */
    STM32_AudioRec_NVIC_Init();
    
    /* Configure the ADC */
    STM32_AudioRec_ADC_Init();
    
    /* Configure the Timer which triggers the ADC */
    STM32_AudioRec_TIM_Init(AudioFreq);
    
    /* Initialize the wave file header */
    //STM32_AudioRec_HeaderInit();
    
    /* Set the local parameters */
    AudioRecBitRes = BitRes;
    AudioRecChnlNbr = ChnlNbr;
    
    /* Set state of the audio recorder to initialized */
    AudioRecInited = 1;
    
    /* Return 0 if all operations are OK */
    return 0;
  }
}

/**
  * @brief  Start audio recording
  * @param  None
  * @retval None
  */
uint32_t STM32_AudioRec_Start(uint8_t* pbuf, uint32_t size)
{
  /* Check if the interface has already been initialized */
  if (AudioRecInited)
  {
    /* Store the location and size of the audio buffer */
    pAudioRecBuf = pbuf;
    AudioRecCurrSize = size;
    
    /*
      ...
      To be implemented
      ...
    */
    
    /* Return 0 if all operations are OK */
    return 0;
  }
  else
  {
    /* Cannot perform operation */
    return 1;
  }
}

/**
  * @brief  Stop audio recording
  * @param  None
  * @retval None
  */
uint32_t STM32_AudioRec_Stop(void)
{
  /* Check if the interface has already been initialized */
  if (AudioRecInited)
  {
    /*
      ...
      To be implemented
      ...
    */
    
    /* Return 0 if all operations are OK */
    return 0;
  }
  else
  {
    /* Cannot perform operation */
    return 1;
  }
}

/**
  * @brief  Update the current value of the timer auto reload
  * @param  Step: decrement/increment step
  * @param  Direction: Direction of increment/decrement. Can be 
  *         AUDIO_REC_DIRECTION_DOWN or AUDIO_REC_DIRECTION_UP.
  * @retval None
  */
uint32_t STM32_AudioRec_ClkUpdate(uint32_t Step, uint32_t Direction)
{    
  /* Check if the interface has already been initialized */
  if (AudioRecInited == 0)
  {
    /* Operation cannot be performed */
    return 1;
  }
  
  /* Check on the direction of the clock update */
  if (Direction == AUDIO_REC_DIRECTION_UP)
  {
    /* Speed up the sampling rate */
    AutoReload  -= Step;
  }
  else
  {
    /* Slow down the sampling rate */
    AutoReload  += Step;
    
  }
  
  /* If operation is OK, return 0 */
  return 0;
}

/**
  * @brief  Initialize the wave header file
  * @param  None
  * @retval None
  */
uint32_t STM32_AudioRec_HeaderInit(uint8_t* pHeadBuf)
{
  uint16_t count = 0;

  /* write chunkID, must be 'RIFF'  ----------------------------------------------*/
  pHeadBuf[0] = 'R';
  pHeadBuf[1] = 'I';
  pHeadBuf[2] = 'F';
  pHeadBuf[3] = 'F';

  /* Write the file length ----------------------------------------------------*/
  /* The sampling time 8000 Hz
   To recorde 30s we need 8000 x 30 x 1 (8-Bit data) = 661500 =0xA17FC*/
  pHeadBuf[4] = 0xFC;
  pHeadBuf[5] = 0x17;
  pHeadBuf[6] = 0x0A;
  pHeadBuf[7] = 0x00;

  /* Write the file format, must be 'WAVE' ------------------------------------*/
  pHeadBuf[8]  = 'W';
  pHeadBuf[9]  = 'A';
  pHeadBuf[10] = 'V';
  pHeadBuf[11] = 'E';

  /* Write the format chunk, must be'fmt ' --------------------------------------*/
  pHeadBuf[12]  = 'f';
  pHeadBuf[13]  = 'm';
  pHeadBuf[14]  = 't';
  pHeadBuf[15]  = ' ';

  /* Write the length of the 'fmt' data, must be 0x10 -------------------------*/
  pHeadBuf[16]  = 0x10;
  pHeadBuf[17]  = 0x00;
  pHeadBuf[18]  = 0x00;
  pHeadBuf[19]  = 0x00;

  /* Write the audio format, must be 0x01 (PCM) -------------------------------*/
  pHeadBuf[20]  = 0x01;
  pHeadBuf[21]  = 0x00;

  /* Write the number of channels, must be 0x01 (Mono) ------------------------*/
  pHeadBuf[22]  = 0x01;
  pHeadBuf[23]  = 0x00;

  /* Write the Sample Rate 22050Hz---------------------------------------------*/
  /* Write Little Endian 22050 = 0x5622   0x1F40*/
  pHeadBuf[24]  = (uint8_t)((REC_FREQ & 0xFF));//0x40;
  pHeadBuf[25]  = (uint8_t)((REC_FREQ >> 8) & 0xFF);//0x1F;
  pHeadBuf[26]  = (uint8_t)((REC_FREQ >> 16) & 0xFF);//0x00;
  pHeadBuf[27]  = (uint8_t)((REC_FREQ >> 24) & 0xFF);//0x00;

  /* Write the Byte Rate ------------------------------------------------------*/
  pHeadBuf[28]  = (uint8_t)((REC_FREQ & 0xFF));//0x40; /*0x22*/
  pHeadBuf[29]  = (uint8_t)((REC_FREQ >> 8) & 0xFF);//0x1F; /*0x56*/
  pHeadBuf[30]  = (uint8_t)((REC_FREQ >> 16) & 0xFF);//0x00;
  pHeadBuf[31]  = (uint8_t)((REC_FREQ >> 24) & 0xFF);//0x00;

  /* Write the block alignment ------------------------------------------------*/
  pHeadBuf[32]  = 0x01;/*0x02*/
  pHeadBuf[33]  = 0x00;

  /* Write the number of bits per sample --------------------------------------*/
  pHeadBuf[34]  = 0x08; /*0x10*/
  pHeadBuf[35]  = 0x00;

  /* Write the Data chunk, must be 'data' ---------------------------------------*/
  pHeadBuf[36]  = 'd';
  pHeadBuf[37]  = 'a';
  pHeadBuf[38]  = 't';
  pHeadBuf[39]  = 'a';

  /* Write the number of sample data ------------------------------------------*/
  pHeadBuf[40]  = 0xFC;
  pHeadBuf[41]  = 0x17;
  pHeadBuf[42]  = 0x0A;
  pHeadBuf[43]  = 0x00;

  /* Fill the missing bytes in Buffer1 with 0x80 */
  for (count = 44; count < 512 ; count ++)
  {
    pHeadBuf[count] = 0x80;
  }
  
  /* Return 0 if all operations are OK */
  return 0;
}

/**
  * @brief  This function handles AUDIO_REC_TIM global interrupt request.
  * @param  None
  * @retval None
  */
void AUDIO_REC_TIM_IRQHANDLER(void)
{
    /*
      ...
      To be implemented
      ...
    */
}




/**
  * @brief  STM32_AudioRec_GPIO_Init.
  * @param  None
  * @retval None
  */
static void STM32_AudioRec_GPIO_Init(void)
{
    /*
      ...
      To be implemented
      ...
    */
}

/**
  * @brief  STM32_AudioRec_ADC_Init.
  * @param  None
  * @retval None
  */
static void STM32_AudioRec_ADC_Init(void)
{
    /*
      ...
      To be implemented
      ...
    */ 
}

/**
  * @brief  STM32_AudioRec_TIM_Init.
  * @param  None
  * @retval None
  */
static void STM32_AudioRec_TIM_Init(uint32_t AudioFreq)
{
    /*
      ...
      To be implemented
      ...
    */
}

/**
  * @brief  STM32_AudioRec_NVIC_Init.
  * @param  None
  * @retval None
  */
static void STM32_AudioRec_NVIC_Init(void)
{
    /*
      ...
      To be implemented
      ...
    */
}

/**
  * @}
  */

/**
  * @}
  */
/**
  * @}
  */

/******************* (C) COPYRIGHT 2012 STMicroelectronics *****END OF FILE****/
